"""Abstract base interface for vehicle data sources."""
from __future__ import print_function

import threading
import logging
import string
import sys
import datetime

from openxc.formats.binary import ProtobufStreamer, ProtobufFormatter
from openxc.formats.json import JsonStreamer, JsonFormatter

LOG = logging.getLogger(__name__)

class MissingPayloadFormatError(Exception): pass

class DataSource(threading.Thread):
    """Interface for all vehicle data sources. This inherits from Thread and
    when a source is added to a vehicle it attempts to call the ``start()``
    method if it exists. If an implementer of DataSource needs some background
    process to read data, it's just a matter of defining a ``run()`` method.

    A data source requires a callback method to be specified. Whenever new data
    is received, it will pass it to that callback.
    """
    def __init__(self, callback=None, log_mode=None, payload_format=None):
        """Construct a new DataSource.

        By default, DataSource threads are marked as ``daemon`` threads, so they
        will die as soon as all other non-daemon threads in the process have
        quit.

        Kwargs:
            callback - function to call with any new data received
        """
        super(DataSource, self).__init__()
        self.callback = callback
        self.daemon = True
        self.running = True
        self._streamer = None
        self._formatter = None
        self.format = payload_format

        self.logger = SourceLogger(self, log_mode)

    @property
    def format(self):
        return self._format

    @format.setter
    def format(self, value):
        self._format = value
        if value == "json":
            self.streamer = JsonStreamer()
            self.formatter = JsonFormatter
        elif value == "protobuf":
            self.streamer = ProtobufStreamer()
            self.formatter = ProtobufFormatter

    @property
    def streamer(self):
        if self._streamer is None:
            raise MissingPayloadFormatError("Unable to auto-detect payload "
                "format, must specify manually with --format [json|protobuf]")
        return self._streamer

    @streamer.setter
    def streamer(self, value):
        self._streamer = value

    @property
    def formatter(self):
        if self._formatter is None:
            raise MissingPayloadFormatError("Unable to auto-detect payload "
                "format, must specify manually with --format [json|protobuf]")
        return self._formatter

    @formatter.setter
    def formatter(self, value):
        self._formatter = value

    @property
    def bytes_received(self):
        return self.streamer.bytes_received

    def start(self):
        self.logger.start()
        super(DataSource, self).start()

    def stop(self):
        self.logger.stop()
        self.running = False

    def read(self, timeout=None):
        """Read data from the source.

        Kwargs:
            timeout (float) - if the source implementation could potentially
                block, timeout after this number of seconds.
        """
        raise NotImplementedError("Don't use DataSource directly")

    def read_logs(self, timeout=None):
        """Read log data from the source.

        Kwargs:
            timeout (float) - if the source implementation could potentially
                block, timeout after this number of seconds.
        """
        raise NotImplementedError("Don't use DataSource directly")


class SourceLogger(threading.Thread):
    FILENAME_TEMPLATE = "%d-%m-%Y.%H-%M-%S"

    def __init__(self, source, mode="off"):
        super(SourceLogger, self).__init__()
        self.daemon = True
        self.source = source
        self.mode = mode
        self.file = None
        self.running = True

        if self.mode == "file":
            filename = "openxc-logs-%s.txt" % datetime.datetime.now().strftime(
                    self.FILENAME_TEMPLATE)
            self.file = open(filename, 'wa')

    def stop(self):
        self.running = False

    def record(self, message):
        if self.mode is not None and self.mode != "off" and len(message) > 0:
            log_file = None
            if self.mode == "stderr":
                log_file = sys.stderr
            elif self.mode == "file" and self.file is not None:
                log_file = self.file
            print("LOG: %s" % message, file=log_file)

    def run(self):
        """Continuously read data from the source and attempt to parse a valid
        message from the buffer of bytes. When a message is parsed, passes it
        off to the callback if one is set.
        """
        message_buffer = b""
        while self.running:
            try:
                message_buffer += self.source.read_logs()
            except DataSourceError as e:
                if self.running:
                    LOG.warn("Can't read logs from data source -- stopping: %s", e)
                break
            except NotImplementedError as e:
                LOG.info("%s doesn't support logging" % self)
                break

            while True:
                if "\x00" not in message_buffer:
                    break
                record, _, remainder = message_buffer.partition(b"\x00")
                self.record(record)
                message_buffer = remainder


class BytestreamDataSource(DataSource):
    """A source that receives data is a series of bytes, with discrete messages
    separated by a newline character.

    Subclasses of this class need only to implement the ``read`` method.
    """

    def __init__(self, **kwargs):
        super(BytestreamDataSource, self).__init__(**kwargs)
        self.corrupted_messages = 0
        self.running = True

    def _message_valid(self, message):
        if not hasattr(message, '__iter__'):
            return False
        if not ('name' in message and 'value' in message or
                    ('id' in message and 'data' in message) or
                    ('id' in message and 'bus' in message) or
                    'command_response' in message):
            return False
        return True

    def run(self):
        """Continuously read data from the source and attempt to parse a valid
        message from the buffer of bytes. When a message is parsed, passes it
        off to the callback if one is set.
        """
        while self.running:
            try:
                payload = self.read()
            except DataSourceError as e:
                if self.running:
                    LOG.warn("Can't read from data source -- stopping: %s", e)
                break

            try:
                self.streamer
            except MissingPayloadFormatError:
                json_chars = ['\x00']
                json_chars.extend(string.printable)
                if all((char in json_chars for char in payload)):
                    self.format = "json"
                else:
                    self.format = "protobuf"
            self.streamer.receive(payload)

            while True:
                message = self.streamer.parse_next_message()
                if message is None:
                    break

                if not self._message_valid(message):
                    self.corrupted_messages += 1
                    break

                if self.callback is not None:
                    self.callback(message)
                self._receive_command_response(message)

    def _receive_command_response(self, message):
        # TODO the controller/source are getting a litlte mixed up since the
        # controller now needs to receive responses from the soruce side, maybe
        # just mix them again. the only exception to being both is a trace
        # source, and we can just leave the controller methods on that
        # unimplemented
        self.open_requests = getattr(self, 'open_requests', [])
        for open_request in self.open_requests:
            open_request.put(message)


class DataSourceError(Exception):
    pass



"""
This module contains the methods for the ``openxc-dump`` command line program.

`main` is executed when ``openxc-dump`` is run, and all other callables in this
module are internal only.
"""
from __future__ import absolute_import

import argparse
import time
import logging

from openxc.formats.json import JsonFormatter
from .common import device_options, configure_logging, select_device

def receive(message, **kwargs):
    message['timestamp'] = time.time()
    print(JsonFormatter.serialize(message))


def parse_options():
    parser = argparse.ArgumentParser(
            description="View a raw OpenXC data stream",
            parents=[device_options()])

    arguments = parser.parse_args()
    return arguments


def main():
    configure_logging(logging.DEBUG)
    arguments = parse_options()

    source_class, source_kwargs = select_device(arguments)
    source = source_class(callback=receive, **source_kwargs)
    source.start()
    # TODO test this, I'd prefer it to the sleep loop
    source.join()



"""Data containers and other utilities."""
import time
import json
import collections
import logging
import os
import sys

LOG = logging.getLogger(__name__)

class Range(object):
    """Encapsulates a ranged defined by a min and max numerical value."""
    def __init__(self, minimum, maximum):
        self.min = minimum
        self.max = maximum

    def within_range(self, value):
        """Returns True if the value is between this Range, inclusive."""
        return value >= self.min and value <= self.max

    @property
    def spread(self):
        """Returns the spread between this Range's min and max."""
        return self.max - self.min


class AgingData(object):
    """Mixin to associate a class with a time of birth."""

    def __init__(self):
        self.created_at = time.time()

    @property
    def age(self):
        """Return the age of the data in seconds."""
        return time.time() - self.created_at


def quacks_like_dict(object):
    """Check if object is dict-like"""
    return isinstance(object, collections.Mapping)


def quacks_like_list(object):
    """Check if object is list-like"""
    return hasattr(object, '__iter__') and hasattr(object, 'append')


def merge(a, b):
    """Merge two deep dicts non-destructively

    Uses a stack to avoid maximum recursion depth exceptions

    >>> a = {'a': 1, 'b': {1: 1, 2: 2}, 'd': 6}
    >>> b = {'c': 3, 'b': {2: 7}, 'd': {'z': [1, 2, 3]}}
    >>> c = merge(a, b)
    >>> from pprint import pprint; pprint(c)
    {'a': 1, 'b': {1: 1, 2: 7}, 'c': 3, 'd': {'z': [1, 2, 3]}}
    """
    assert quacks_like_dict(a), quacks_like_dict(b)
    dst = a.copy()

    stack = [(dst, b)]
    while stack:
        current_dst, current_src = stack.pop()
        for key in current_src:
            if key not in current_dst:
                current_dst[key] = current_src[key]
            else:
                if (quacks_like_dict(current_src[key]) and
                        quacks_like_dict(current_dst[key])):
                    stack.append((current_dst[key], current_src[key]))
                elif (quacks_like_list(current_src[key]) and
                        quacks_like_list(current_dst[key])):
                    current_dst[key].extend(current_src[key])
                else:
                    current_dst[key] = current_src[key]
    return dst


def find_file(filename, search_paths):
    for search_path in search_paths:
        if filename[0] != '/':
            full_path = "%s/%s" % (search_path, filename)
        else:
            full_path = filename
        if os.path.exists(full_path):
            return full_path
    fatal_error("Unable to find '%s' in search paths (%s)" % (
            filename, search_paths))


def load_json_from_search_path(filename, search_paths):
    with open(find_file(filename, search_paths)) as json_file:
        try:
            data = json.load(json_file)
        except ValueError as e:
            fatal_error("%s does not contain valid JSON: \n%s\n" %
                    (filename, e))
        else:
            return data

def fatal_error(message):
    LOG.error(message)
    sys.exit(1)



"""
This module contains the methods for the ``openxc-control`` command line
program.

`main` is executed when ``openxc-control`` is run, and all other callables in
this module are internal only.
"""
from __future__ import absolute_import

import argparse
import sys
import time

from openxc.formats.json import JsonFormatter
from .common import device_options, configure_logging, select_device


def version(interface):
    print("Device is running version %s" % interface.version())

def sd_mount_status(interface):
    result = interface.sd_mount_status()
    if(result == 1):
        print("SD card mount status: true")
    else:
        print("SD card mount status: false")		

def device_id(interface):
    print("Device ID is %s" % interface.device_id())

def passthrough(interface, bus, passthrough_enabled):
    if interface.set_passthrough(bus, passthrough_enabled):
        print("Bus %u passthrough set to %s" % (bus, passthrough_enabled))

def af_bypass(interface, bus, bypass):
    if interface.set_acceptance_filter_bypass(bus, bypass):
        if bypass:
            bypass_string = "bypassed"
        else:
            bypass_string = "enabled"
        print("Bus %u AF is now %s" % (bus, bypass_string))

def set_payload_format(interface, payload_format):
    if interface.set_payload_format(payload_format):
        print("Changed payload format to %s" % payload_format)

def set_rtc_time(interface, unix_time):
    if interface.rtc_configuration(unix_time):
        print("Time set to %d" % unix_time)

def modem_configuration(interface, host, port):
    if interface.modem_configuration(host, port):
        print("host set to %s:%s" %(host, port))

def write_file(interface, filename):
    first_timestamp = None
    with open(filename, "r") as output_file:
        corrupt_entries = 0
        message_count = 0
        start_time = time.time()
        for line in output_file:
            try:
                parsed_message = JsonFormatter.deserialize(line.encode("utf-8"))
                if not isinstance(parsed_message, dict):
                    raise ValueError()
            except ValueError:
                corrupt_entries += 1
            else:
                # TODO at the moment it's taking longer to write all of
                # individual CAN messages than the time that actually
                # elapsed in receiving the trace - need to implement
                # batching to speed this up. right now this will never sleep
                # because it's always behind.
                timestamp = parsed_message.get('timestamp', None)
                # TODO this duplicates some code from sources/trace.py
                if timestamp is not None:
                    first_timestamp = first_timestamp or timestamp
                    target_time = start_time + (timestamp - first_timestamp)
                    time.sleep(max(.0002, target_time - time.time()))

                message_count += 1
                interface.write(**parsed_message)
        print("%d lines sent" % message_count)
        if corrupt_entries > 0:
            print("%d invalid lines in the data file were not sent" %
                    corrupt_entries)


def parse_options():
    parser = argparse.ArgumentParser(description="Send control messages to an "
            "attached OpenXC vehicle interface", parents=[device_options()])
    parser.add_argument("command", type=str,
            choices=['version', 'write', 'id', 'set', 'sd_mount_status'])
    write_group = parser.add_mutually_exclusive_group()
    write_group.add_argument("--name", action="store", dest="write_name",
            help="name for message write request")
    write_group.add_argument("--id", action="store", dest="write_id",
            help="ID for raw message write request")
    parser.add_argument("--bus", action="store", dest="bus",
            default=1,
            help="CAN bus number for the control request")
    parser.add_argument("--value", action="store", dest="write_value",
            help="optional value for message write request")
    parser.add_argument("--event", action="store", dest="write_event",
            help="optional event for message write request")
    parser.add_argument("--data", action="store", dest="write_data",
            help="data for raw message write request")
    parser.add_argument("--frame-format", action="store",
            dest="write_frame_format",  choices=['standard', 'extended'],
            help="explicit frame format for raw message write request")
    write_group.add_argument("-f", "--write-input-file", action="store",
            dest="write_input_file",
            help="the path to a file with a list of raw or translated "
                    "messages to write to the selected vehicle interface")
    parser.add_argument("--passthrough", action="store_true", default=None,
            dest="passthrough_enabled")
    parser.add_argument("--no-passthrough", action="store_false", default=None,
            dest="passthrough_enabled")
    parser.add_argument("--af-bypass", action="store_true", default=None,
            dest="af_bypass")
    parser.add_argument("--no-af-bypass", action="store_false", default=None,
            dest="af_bypass")
    parser.add_argument("--new-payload-format", action="store", default=None,
            choices=['json', 'protobuf'], dest="new_payload_format")
    parser.add_argument("--time", action="store",default=None,
            dest="unix_time")
    parser.add_argument("--host", action="store", default=None,
            dest="host")
    parser.add_argument("--port", action="store", default=80,
            dest="port")
    return parser.parse_args()


def main():
    configure_logging()
    arguments = parse_options()

    interface_class, interface_kwargs = select_device(arguments)
    interface = interface_class(**interface_kwargs)
    interface.start()

    if arguments.command == "version":
        version(interface)
    elif arguments.command == "sd_mount_status":
        sd_mount_status(interface)
    elif arguments.command == "id":
        device_id(interface)
    elif arguments.command == "set":
        if arguments.passthrough_enabled is not None:
            passthrough(interface, int(arguments.bus), arguments.passthrough_enabled)
        if arguments.af_bypass is not None:
            af_bypass(interface, int(arguments.bus), arguments.af_bypass)
        if arguments.new_payload_format is not None:
            set_payload_format(interface, arguments.new_payload_format)
        if arguments.unix_time is not None:
            set_rtc_time(interface, int(arguments.unix_time))
        if arguments.host is not None:
            modem_configuration(interface, host, port)
    elif arguments.command.startswith("write"):
        if arguments.command == "write":
            if arguments.write_name:
                interface.write(name=arguments.write_name,
                        value=arguments.write_value,
                        event=arguments.write_event)
            elif arguments.write_id:
                if not arguments.write_data:
                    sys.exit("%s requires an id and data" % arguments.command)
                # TODO we should use unhexlify as with the diagnostic command
                # payloads so we can standardize the API and not deal with hex
                # strings in code
                interface.write(bus=int(arguments.bus),
                        id=arguments.write_id,
                        data=arguments.write_data,
                        frame_format=arguments.write_frame_format)
            elif arguments.write_input_file:
                write_file(interface, arguments.write_input_file)
            else:
                sys.exit("%s requires a signal name, message ID or filename" % arguments.command)
    else:
        print("Unrecognized command \"%s\"" % arguments.command)



class VehicleMessageStreamer(object):
    message_buffer = b""
    bytes_received = 0

    def receive(self, payload):
        if not isinstance(payload, bytes):
            payload = payload.encode("utf-8")
        if len(payload) > 0:
            self.message_buffer += payload
            self.bytes_received += len(payload)



"""Common functions for parsing command line arguments, shared with all openxc
command line utilities.
"""
import argparse
import logging

from openxc.sources.trace import TraceDataSource
from openxc.interface import SerialVehicleInterface, UsbVehicleInterface, \
        NetworkVehicleInterface, BluetoothVehicleInterface


def device_options():
    parser = argparse.ArgumentParser(add_help=False)
    device_group = parser.add_mutually_exclusive_group()
    device_group.add_argument("--usb", action="store_true", dest="use_usb",
            default=True,
            help="use a USB-connected VI as the data source")
    device_group.add_argument("--serial", action="store_true",
            dest="use_serial",
            help="use a serial-connected VI as the data source")
    device_group.add_argument("--bluetooth", action="store_true",
            dest="use_bluetooth",
            help="use a Bluetooth-connected VI as the data source")
    device_group.add_argument("--network", action="store_true",
            dest="use_network",
            help="use a network-connected VI as the data source")
    device_group.add_argument("--trace", action="store", dest="trace_file",
            help="use a pre-recorded OpenXC JSON trace file as the data source")
    parser.add_argument("--usb-vendor",
            action="store",
            dest="usb_vendor",
            help="USB vendor ID for the VI")
    parser.add_argument("--usb-product",
            action="store",
            dest="usb_product",
            help="USB product ID for the VI")
    parser.add_argument("--bluetooth-address",
            action="store",
            dest="bluetooth_address",
            help="MAC address of Bluetooth VI. If not provided, will " +
                    "perform a scan and select first device with name " +
                    "matching \"OpenXC-VI-*\"")
    parser.add_argument("--serial-port",
            action="store",
            dest="serial_port",
            help="virtual COM port path for serial VI")
    parser.add_argument("--serial-baudrate",
            action="store",
            dest="baudrate",
            help="baudrate for serial-connected VI")
    parser.add_argument("--network-host",
            action="store",
            dest="network_host",
            help="host for networked VI")
    parser.add_argument("--network-port",
            action="store",
            dest="network_port",
            help="network port for networked VI")
    parser.add_argument("--log-mode",
            action="store",
            default="off",
            choices=["off", "stderr", "file"],
            dest="log_mode",
            help="record logs to a file or stderr, if available from the interface")
    parser.add_argument("--format",
            action="store",
            choices=["json", "protobuf"],
            dest="format",
            help="select the data format for sending and receiving with the VI")
    return parser


def configure_logging(level=logging.WARN):
    logging.getLogger("openxc").addHandler(logging.StreamHandler())
    logging.getLogger("openxc").setLevel(level)

def select_device(arguments):
    if arguments.use_serial:
        source_class = SerialVehicleInterface
        source_kwargs = dict(port=arguments.serial_port,
                baudrate=arguments.baudrate)
    elif arguments.trace_file:
        source_class = TraceDataSource
        source_kwargs = dict(filename=arguments.trace_file)
    elif arguments.use_network:
        source_class = NetworkVehicleInterface
        source_kwargs = dict(host=arguments.network_host,
                port=arguments.network_port)
    elif arguments.use_bluetooth:
        source_class = BluetoothVehicleInterface
        source_kwargs = dict(address=arguments.bluetooth_address)
    else:
        source_class = UsbVehicleInterface
        source_kwargs = dict(vendor_id=arguments.usb_vendor,
                product_id=arguments.usb_product)

    source_kwargs['log_mode'] = arguments.log_mode
    source_kwargs['payload_format'] = arguments.format
    return source_class, source_kwargs



"""JSON formatting utilities."""
from __future__ import absolute_import

import json

from openxc.formats.base import VehicleMessageStreamer

class JsonStreamer(VehicleMessageStreamer):
    SERIALIZED_COMMAND_TERMINATOR = b"\x00"

    def parse_next_message(self):
        parsed_message = None
        remainder = self.message_buffer
        message = ""
        if self.SERIALIZED_COMMAND_TERMINATOR in self.message_buffer:
            message, _, remainder = self.message_buffer.partition(
                    self.SERIALIZED_COMMAND_TERMINATOR)
            try:
                parsed_message = JsonFormatter.deserialize(message)
                if not isinstance(parsed_message, dict):
                    raise ValueError()
            except ValueError:
                pass
        self.message_buffer = remainder
        return parsed_message

    def serialize_for_stream(self, message):
        return JsonFormatter.serialize(
                message) + self.SERIALIZED_COMMAND_TERMINATOR

class JsonFormatter(object):

    @classmethod
    def deserialize(cls, message):
        return json.loads(message.decode("utf8"))

    @classmethod
    def serialize(cls, data):
        return json.dumps(data).encode("utf8")

    @classmethod
    def _validate(cls, message):
        """Confirm the validitiy of a given dict as an OpenXC message.

        Returns:
            ``True`` if the message contains at least a ``name`` and ``value``.
        """
        valid = False
        if(('name' in message and 'value' in message) or
                ('id' in message and 'data' in message)):
            valid = True
        return valid



"""
This module contains the methods for the ``openxc-gps`` command line program.

`main` is executed when ``openxc-gps`` is run, and all other callables in this
module are internal only.
"""
from __future__ import absolute_import

import argparse
from xml.etree import ElementTree as ET

from openxc.sources.trace import TraceDataSource
from .common import device_options, configure_logging, select_device


class GPXTranscoder(object):
    def __init__(self):
        self.root = ET.Element("gpx")
        track = ET.SubElement(self.root, "trk")
        number = ET.SubElement(track, "number")
        number.text = "1"
        self.segment = ET.SubElement(track, "trkseg")
        self.latitude = self.longitude = None

    def output(self):
        return ET.tostring(ET.ElementTree(self.root).getroot())

    def receive(self, message, **kwargs):
        if message['name'] == 'latitude':
            self.latitude = message['value']
        elif message['name'] == 'longitude':
            self.longitude = message['value']

        if self.latitude and self.longitude:
            point = ET.SubElement(self.segment, "trkpt")
            point.set('lat', str(self.latitude))
            point.set('lon', str(self.longitude))
            self.latitude = self.longitude = None


def parse_options():
    parser = argparse.ArgumentParser(description=
            "Create a GPS trace in various formats from OpenXC input data")
    parser.add_argument("trace_file", metavar ='TRACEFILE',
            help="trace file to pull GPX log from")
    parser.add_argument("-f", "--format", type=str, choices=['gpx'],
            default='gpx', help="select the output format of the GPS trace")
    arguments = parser.parse_args()
    return arguments


def main():
    configure_logging()
    arguments = parse_options()

    transcoder = GPXTranscoder()

    source = TraceDataSource(transcoder.receive, filename=arguments.trace_file,
            loop=False, realtime=False)
    source.start()
    source.join()

    print(transcoder.output().decode("utf-8"))



import logging

from openxc.utils import fatal_error
from .structures import Signal, Message

LOG = logging.getLogger(__name__)

class Network(object):
    """Represents all the messages on a single bus in an XML-backed database."""

    def __init__(self, database_name, tree, all_messages):
        self.messages = {}

        for message_id, message in all_messages.items():
            numeric_message_id = int(message_id, 0)
            query = "./Node/TxMessage[ID=\"0x%s\"]"
            # Search for both lower and upper case hex
            for attr_value in ["%X", "%x"]:
                node = tree.find(query % (attr_value % numeric_message_id))
                if node is not None:
                    break
            if node is None:
                LOG.warning("Unable to find message ID 0x%x in %s" % (
                    numeric_message_id, database_name))
            else:
                if 'signals' not in message:
                    fatal_error("This message object is missing a "
                        "top-level 'signals' field: %s" % message)
                self.messages[numeric_message_id] = XMLBackedMessage.from_xml_node(
                    node, message_id, message['signals'])

    def to_dict(self):
        return {'messages': dict(("%s" % message.string_id,
                    message.to_dict())
                for message in list(self.messages.values())
                if len(message.signals) > 0)}

class XMLBackedMessage(Message):

    @classmethod
    def from_xml_node(cls, node, string_message_id, mapped_signals):
        message = cls()

        message.name = node.find("Name").text
        # Keep around whatever string is used in the mapping, so our dict merge
        # works with case insensitive key matching
        message.string_id = string_message_id
        message.id = int(node.find("ID").text, 0)

        for signal_name, mapped_signal in mapped_signals.iteritems():
            mapped_signal_node = node.find("Signal[Name=\"%s\"]" % signal_name)
            if mapped_signal_node is not None:
                signal = XMLBackedSignal.from_xml_node(mapped_signal_node)
                signal.generic_name = mapped_signal.get('generic_name', None) or signal_name
                message.signals[signal_name] = signal

        return message

class XMLBackedSignal(Signal):

    @classmethod
    def from_xml_node(cls, node):
        """Construct a Signal instance from an XML node exported from a Vector
        CANoe .dbc file."""
        return cls(name=node.find("Name").text,
                bit_position=int(node.find("Bitposition").text),
                bit_size=int(node.find("Bitsize").text),
                factor=float(node.find("Factor").text),
                offset=float(node.find("Offset").text),
                min_value=float(node.find("Minimum").text),
                max_value=float(node.find("Maximum").text))


def parse_database(database_filename):
    try:
      from lxml import etree
    except ImportError:
      LOG.warning("Install the 'lxml' Python package to speed up CAN database parsing")
      try:
        # Python 2.5
        import xml.etree.cElementTree as etree
      except ImportError:
        try:
          # Python 2.5
          import xml.etree.ElementTree as etree
        except ImportError:
          try:
            # normal cElementTree install
            import cElementTree as etree
          except ImportError:
            try:
              # normal ElementTree install
              import elementtree.ElementTree as etree
            except ImportError:
              fatal_error("Failed to import ElementTree from any known place")
    return etree.parse(database_filename)


def merge_database_into_mapping(database_filename, database_tree, messages):
    if len(messages) == 0:
        LOG.warning("No messages specified for mapping from XML")
        return {}
    else:
        return Network(database_filename, database_tree, messages).to_dict()



import logging

try:
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass

logging.getLogger("openxc").addHandler(NullHandler())



"""Trace file recording operations."""
from threading import Thread
import datetime
import time

from openxc.formats import JsonFormatter
from .queued import QueuedSink


class FileRecorderSink(QueuedSink):
    """A sink to record trace files based on the messages received from all data
    sources.
    """
    FILENAME_DATE_FORMAT = "%Y-%m-%d-%H"
    FILENAME_FORMAT = "%s.json"

    def __init__(self):
        super(FileRecorderSink, self).__init__()
        self.recorder = self.Recorder(self.queue)

    class Recorder(Thread):
        def __init__(self, queue):
            super(FileRecorderSink.Recorder, self).__init__()
            self.daemon = True
            self.queue = queue
            self.start()

        @staticmethod
        def _generate_filename():
            current_date = datetime.datetime.now()
            return FileRecorderSink.FILENAME_FORMAT % current_date.strftime(
                    FileRecorderSink.FILENAME_DATE_FORMAT)

        def run(self):
            while True:
                last_hour_opened = datetime.datetime.now().hour
                filename = self._generate_filename()
                with open(filename, 'a') as output_file:
                    while True:
                        message, _ = self.queue.get()
                        message['timestamp'] = time.time()
                        output_file.write(JsonFormatter.serialize(message))
                        self.queue.task_done()

                        if datetime.datetime.now().hour > last_hour_opened:
                            break



"""
This module contains the methods for the ``openxc-obd2scanner`` command line
program.

`main` is executed when ``openxc-obd2scanner`` is run, and all other callables in this
module are internal only.
"""
from __future__ import absolute_import

import argparse

from .common import device_options, configure_logging, select_device

def scan(controller, bus=None):

    # TODO could read the response from the "PIDs supported" requests to see
    # what the vehicle reports that it *should* support.
    print("Beginning sequential scan of all OBD-II PIDs")
    for pid in range(0, 0x88):
        response = controller.create_diagnostic_request(0x7df, mode=0x1, bus=bus,
                wait_for_first_response=True, pid=pid)
        if response is not None:
            print("PID 0x%x responded with: %s" % (pid, response))
        else:
            print("PID 0x%x did not respond" % pid)

def parse_options():
    parser = argparse.ArgumentParser(description="Send requests for all "
                "OBD-II PIDs sequentially to see what actually responds",
            parents=[device_options()])
    parser.add_argument("--bus")

    return parser.parse_args()


def main():
    configure_logging()
    arguments = parse_options()

    controller_class, controller_kwargs = select_device(arguments)
    controller = controller_class(**controller_kwargs)
    controller.start()

    scan(controller, arguments.bus)



"""A data source for reading from pre-recorded OpenXC trace files."""

from __future__ import absolute_import

import logging
import time

from .base import DataSourceError, BytestreamDataSource

from openxc.formats.json import JsonFormatter

LOG = logging.getLogger(__name__)

class TraceDataSource(BytestreamDataSource):
    """A class to replay a previously recorded OpenXC vehicle data trace file.
    For details on the trace file format, see
    http://openxcplatform.com/android/testing.html.
    """

    def __init__(self, filename=None, realtime=True, loop=True, **kwargs):
        """Construct the source and attempt to open the trace file.

            filename - the full absolute path to the trace file

            realtime - if ``True``, the trace will be replayed at approximately
            the same cadence as it was recorded. Otherwise, the trace file
            will be replayed as fast as possible (likely much faster than
            any vehicle).

            loop - if ``True``, the trace file will be looped and will provide
            data until the process exist or the source is stopped.
        """
        super(TraceDataSource, self).__init__(**kwargs)
        self.realtime = realtime
        self.loop = loop
        self.filename = filename
        self._reopen_file()

    def _reopen_file(self):
        if getattr(self, 'trace_file', None) is not None:
            self.trace_file.close()
        self.trace_file = self._open_file(self.filename)
        self.starting_time = time.time()

    def _store_timestamp(self, timestamp):
        """If not already saved, cache the first timestamp in the active trace
        file on the instance.
        """
        if getattr(self, 'first_timestamp', None) is None:
            self.first_timestamp = timestamp
            LOG.debug("Storing %d as the first timestamp of the trace file %s",
                    self.first_timestamp, self.filename)

    def read(self):
        """Read a line of data from the input source at a time."""
        line = self.trace_file.readline()
        if line == '':
            if self.loop:
                self._reopen_file()
            else:
                self.trace_file.close()
                self.trace_file = None
                raise DataSourceError()

        message = JsonFormatter.deserialize(line)
        timestamp = message.get('timestamp', None)
        if self.realtime and timestamp is not None:
            self._store_timestamp(timestamp)
            self._wait(self.starting_time, self.first_timestamp, timestamp)
        return line + "\x00"

    @staticmethod
    def _open_file(filename):
        """Attempt to open the the file at ``filename`` for reading.

        Raises:
            DataSourceError, if the file cannot be opened.
        """
        if filename is None:
            raise DataSourceError("Trace filename is not defined")

        try:
            trace_file = open(filename, "r")
        except IOError as e:
            raise DataSourceError("Unable to open trace file %s" % filename, e)
        else:
            LOG.debug("Opened trace file %s", filename)
            return trace_file

    @staticmethod
    def _wait(starting_time, first_timestamp, timestamp):
        """Given that the first timestamp in the trace file is
        ``first_timestamp`` and we started playing back the file at
        ``starting_time``, block until the current ``timestamp`` should occur.
        """
        target_time = starting_time + (timestamp - first_timestamp)
        time.sleep(max(target_time - time.time(), 0))



#!/usr/bin/env python
"""
This module is the main entry point for generating vehicle interface source code
from JSON mappings.

Given one or more JSON mapping filenames, and a list of directories where
configuration files may be found, it parses the mappings, validates them, and
prints the resulting source to stdout.
"""

from __future__ import print_function
import sys
import argparse
import logging

from openxc.generator.coder import CodeGenerator
from openxc.generator.message_sets import JsonMessageSet
from openxc.utils import fatal_error, load_json_from_search_path
from .common import configure_logging

LOG = logging.getLogger(__name__)

DEFAULT_SEARCH_PATH = "."


def parse_options():
    parser = argparse.ArgumentParser(description="Generate C++ source code "
            "from CAN signal descriptions in JSON")

    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument("-m", "--message-sets",
            type=str,
            nargs='+',
            dest="message_sets",
            metavar="MESSAGE_SET",
            help="generate source from these JSON-formatted message "
                    "set definitions")
    group.add_argument("--super-set",
            type=str,
            dest="super_set",
            metavar="SUPERSET",
            help="generate source with multiple active message sets, defined in"
                    " this JSON-formatted superset")
    parser.add_argument("-s", "--search-paths",
            type=str,
            nargs='+',
            dest="search_paths",
            metavar="PATH",
            help="add directories to the search path when using relative paths")

    return parser.parse_args()


def main():
    configure_logging()
    arguments = parse_options()

    search_paths = arguments.search_paths or []
    search_paths.insert(0, DEFAULT_SEARCH_PATH)

    message_sets = arguments.message_sets or []
    if arguments.super_set is not None:
        super_set_data = load_json_from_search_path(arguments.super_set,
                arguments.search_paths)
        super_set_message_sets = super_set_data.get('message_sets', [])
        if len(super_set_message_sets) == 0:
            LOG.warning("Superset '%s' has no message sets" %
                    super_set_data.get('name', 'unknown'))
        message_sets.extend(super_set_message_sets)

    generator = CodeGenerator(search_paths)
    for filename in message_sets:
        message_set = JsonMessageSet.parse(filename, search_paths=search_paths,
                skip_disabled_mappings=True)
        if not message_set.validate_messages() or not message_set.validate_name():
            fatal_error("unable to generate code")
        generator.message_sets.append(message_set)

    # TODO dump to a filename or pipe instead
    print(generator.build_source())


if __name__ == "__main__":
    sys.exit(main())



"""Controller implementation for an OpenXC USB device."""
from __future__ import absolute_import

import logging
import usb.core

from .base import Controller, ControllerError

LOG = logging.getLogger(__name__)


class UsbControllerMixin(Controller):
    """An implementation of a Controller type that connects to an OpenXC USB
    device.

    This class acts as a mixin, and expects ``self.device`` to be an instance
    of ``usb.Device``.

    TODO bah, this is kind of weird. refactor the relationship between
    sources/controllers.
    """
    COMPLEX_CONTROL_COMMAND = 0x83

    def write_bytes(self, data):
        if self.out_endpoint is None:
            LOG.warn("Can't write \"%s\" to USB, OUT endpoint is %x", data,
                    self.out_endpoint)
            return 0
        else:
            return self.out_endpoint.write(data)

    def _send_complex_request(self, request):
        """Send a request via the USB control request endpoint, rather than as a
        bulk transfer.
        """
        self.device.ctrl_transfer(0x40, self.COMPLEX_CONTROL_COMMAND, 0, 0,
                self.streamer.serialize_for_stream(request))

    @property
    def out_endpoint(self):
        """Open a reference to the USB device's only OUT endpoint. This method
        assumes that the USB device configuration has already been set.
        """
        if getattr(self, '_out_endpoint', None) is None:
            config = self.device.get_active_configuration()
            interface_number = config[(0, 0)].bInterfaceNumber
            interface = usb.util.find_descriptor(config,
                    bInterfaceNumber=interface_number)

            self._out_endpoint = usb.util.find_descriptor(interface,
                    custom_match = \
                            lambda e: \
                            usb.util.endpoint_direction(e.bEndpointAddress) == \
                            usb.util.ENDPOINT_OUT)

            if not self._out_endpoint:
                raise ControllerError(
                        "Couldn't find OUT endpoint on the USB device")
        return self._out_endpoint



"""
This module contains the methods for the ``openxc-trace-split`` command line
program.

`main` is executed when ``openxc-trace-split`` is run, and all other callables
in this module are internal only.
"""
from __future__ import absolute_import

import argparse
import datetime
from operator import itemgetter
from collections import defaultdict

from openxc.formats.json import JsonFormatter
from openxc.sources.trace import TraceDataSource
from .common import configure_logging


class BaseSplitter(object):
    def __init__(self):
        self.records = []
        self.buckets = defaultdict(list)

    def _key_for_record(self, record):
        raise NotImplementedError

    def split(self, files):
        for filename in files:
            source = TraceDataSource(self.receive, filename=filename,
                    loop=False, realtime=False)
            source.start()

        self.records.sort(key=itemgetter('timestamp'))
        for record in self.records:
            self.buckets[self._key_for_record(record)].append(record)

        return self.buckets

    def receive(self, message, **kwargs):
        self.records.append(message)


class TimeSplitter(BaseSplitter):
    def __init__(self, unit):
        super(TimeSplitter, self).__init__()
        self.unit = unit

    def _key_for_record(self, record):
        date = datetime.datetime.fromtimestamp(record['timestamp'])
        if self.unit == "day":
            date_format = '%Y-%m-%d'
        elif self.unit == "hour":
            date_format = '%Y-%m-%d-%H'
        return date.strftime(date_format)


class TripSplitter(BaseSplitter):
    # TODO allow overriding this at the command line
    MAXIMUM_RECORD_GAP_SECONDS = 600

    def _key_for_record(self, record):
        timestamp = record['timestamp']
        last_timestamp = getattr(self, 'last_timestamp', None)
        if last_timestamp is not None:
            if (timestamp - last_timestamp) > self.MAXIMUM_RECORD_GAP_SECONDS:
                self.current_trip_key = timestamp
        else:
            self.current_trip_key = timestamp
        self.last_timestamp = timestamp
        return datetime.datetime.fromtimestamp(self.last_timestamp).strftime(
                "%Y-%m-%d-%H.%M")


def parse_options():
    parser = argparse.ArgumentParser(description="Split a collection of "
            "OpenXC trace files by day, hour or trips")
    parser.add_argument("files", action="store", nargs='+', default=False)
    parser.add_argument("-s", "--split", action="store",
            choices=['day', 'hour', 'trip'], default="trip",
            help="select the time unit to split the combined trace files")

    arguments = parser.parse_args()
    return arguments


def main():
    configure_logging()
    arguments = parse_options()

    if arguments.split == 'trip':
        splitter = TripSplitter()
    else:
        splitter = TimeSplitter(arguments.split)

    for key, split in splitter.split(arguments.files).items():
        with open("%s.json" % key, 'w') as output_file:
            for record in split:
                output_file.write(JsonFormatter.serialize(record) + "\n")



"""Define the scalar units used by vehicle measurements."""
from __future__ import absolute_import

import sys
from units import unit, scaled_unit, named_unit
import units.predefined

units.predefined.define_units()


Percentage = unit('%')
Meter = unit('m')
Kilometer = scaled_unit('km', 'm', 1000)
Hour = unit('h')
KilometersPerHour = unit('km') / unit('h')
RotationsPerMinute = unit('rotations') / unit('m')
Litre = unit('L')
Degree = unit('deg')
NewtonMeter = named_unit("Nm", ["N", "m"], [])
MetersPerSecondSquared = unit('m') / (pow(unit('s'), 2))
Undefined = unit('undef')



from __future__ import absolute_import

from .base import BytestreamDataSource, DataSourceError

# TODO can we get rid of this?
import socket


class SocketDataSource(BytestreamDataSource):
    def read(self):
        try:
            line = ""
            while '\x00' not in line:
                # TODO this is fairly inefficient
                line += self.socket.recv(1)
        except (OSError, socket.error, IOError) as e:
            raise DataSourceError("Unable to read from socket connection at  "
                    "%s:%s: %s" % (self.host,self.port, e))
        if not line:
            raise DataSourceError("Unable to read from socket connection at  "
                    "%s:%s" % (self.host,self.port))
        return line

    def write_bytes(self, data):
        return self.socket.send(data)



"""A USB vehicle interface data source."""
from __future__ import absolute_import

import logging
import usb.core
import usb.util

from .base import BytestreamDataSource, DataSourceError

LOG = logging.getLogger(__name__)


class UsbDataSource(BytestreamDataSource):
    """A source to receive data from an OpenXC vehicle interface via USB."""
    DEFAULT_VENDOR_ID = 0x1bc4
    DEFAULT_PRODUCT_ID = 0x0001
    DEFAULT_READ_REQUEST_SIZE = 512

    # If we don't get DEFAULT_READ_REQUEST_SIZE bytes within this number of
    # milliseconds, bail early and return whatever we have - could be zero,
    # could be just less than 512. If data is really pumpin' we can get better
    # throughput if the READ_REQUEST_SIZE is higher, but this delay has to be
    # low enough that a single request isn't held back too long.
    DEFAULT_READ_TIMEOUT = 200

    DEFAULT_INTERFACE_NUMBER = 0
    VEHICLE_DATA_IN_ENDPOINT = 2
    LOG_IN_ENDPOINT = 11

    def __init__(self, vendor_id=None, product_id=None, **kwargs):
        """Initialize a connection to the USB device's IN endpoint.

        Kwargs:
            vendor_id (str or int) - optionally override the USB device vendor
                ID we will attempt to connect to, if not using the OpenXC
                hardware.

            product_id (str or int) - optionally override the USB device product
                ID we will attempt to connect to, if not using the OpenXC
                hardware.

            log_mode - optionally record or print logs from the USB device, which
                are on a separate channel.

        Raises:
            DataSourceError if the USB device with the given vendor ID is not
            connected.
        """
        super(UsbDataSource, self).__init__(**kwargs)
        if vendor_id is not None and not isinstance(vendor_id, int):
            vendor_id = int(vendor_id, 0)
        self.vendor_id = vendor_id or self.DEFAULT_VENDOR_ID

        if product_id is not None and not isinstance(product_id, int):
            product_id = int(product_id, 0)
        self.product_id = product_id or self.DEFAULT_PRODUCT_ID

        devices = usb.core.find(find_all=True, idVendor=self.vendor_id,
                idProduct=self.product_id)
        for device in devices:
            self.device = device
            try:
                self.device.set_configuration()
            except usb.core.USBError as e:
                LOG.warn("Skipping USB device: %s", e)
            else:
                return

        raise DataSourceError("No USB vehicle interface detected - is one plugged in?")

    def read(self, timeout=None):
        return self._read(self.VEHICLE_DATA_IN_ENDPOINT, timeout)

    def read_logs(self, timeout=None):
        return self._read(self.LOG_IN_ENDPOINT, timeout, 64)

    def stop(self):
        super(UsbDataSource, self).stop()
        usb.util.dispose_resources(self.device)

    def _read(self, endpoint_address, timeout=None,
            read_size=DEFAULT_READ_REQUEST_SIZE):
        timeout = timeout or self.DEFAULT_READ_TIMEOUT
        try:
            return self.device.read(0x80 + endpoint_address,
                    read_size, self.DEFAULT_INTERFACE_NUMBER, timeout
                    ).tostring()
        except (usb.core.USBError, AttributeError) as e:
            if e.errno == 110:
                # Timeout, it may just not be sending
                return ""
            raise DataSourceError("USB device couldn't be read", e)



"""
This module contains the methods for the ``openxc-diag`` command line
program.

`main` is executed when ``openxc-diag`` is run, and all other callables in
this module are internal only.
"""
from __future__ import absolute_import

import binascii
import argparse
import time

from .common import device_options, configure_logging, select_device

def diagnostic_request(arguments, controller):
    message = int(arguments.message, 0)
    mode = int(arguments.mode, 0)

    bus = None
    if arguments.bus is not None:
        bus = int(arguments.bus, 0)

    pid = None
    if arguments.pid is not None:
        pid = int(arguments.pid, 0)

    frequency = None
    if arguments.frequency is not None:
        frequency = int(arguments.frequency, 0)

    payload = bytearray()
    if arguments.payload is not None:
        payload = binascii.unhexlify(arguments.payload.split("0x")[1])

    if arguments.command == "add":
        responses = controller.create_diagnostic_request(message, mode, bus=bus,
                pid=pid, frequency=frequency, payload=payload,
                wait_for_first_response=True)
        if len(responses) == 0:
            print("No response received before timeout")
        for response in responses:
            print("Response: %s" % response)
    elif arguments.command == "cancel":
        if controller.delete_diagnostic_request(message, mode, bus=bus,
                pid=pid):
            print("Diagnostic request deleted successfully")
        else:
            print("Error when attempting delete")


def parse_options():
    parser = argparse.ArgumentParser(
            description="Sends a diagnostic message request to a vehicle interface",
            parents=[device_options()])
    parser.add_argument("command", type=str, choices=['add', 'cancel'])
    # TODO need to be able to specify name, factor, offset. Needs to be
    # supported in the controller, too.
    parser.add_argument("--message", required=True, help="CAN message ID for the request")
    parser.add_argument("--mode", required=True, help="Diagnostic mode (or service) number")
    parser.add_argument("--bus", help="CAN bus controller address to send on")
    parser.add_argument("--pid", help="Parameter ID (e.g. for Mode 1 request")
    parser.add_argument("--payload", help="A byte array as a hex string to send as payload, e.g. 0x123")
    parser.add_argument("--frequency", help="Frequency (Hz) to repeat this request. If omitted or 0, it will be a one-time request.")

    return parser.parse_args()


def main():
    configure_logging()
    arguments = parse_options()

    controller_class, controller_kwargs = select_device(arguments)
    controller = controller_class(**controller_kwargs)
    controller.start()

    diagnostic_request(arguments, controller)
    controller.stop()



"""Common operations for all vehicle data sinks."""

class DataSink(object):
    """A base interface for all data sinks. At the minimum, a data sink must
    have a :func:`receive` method.
    """

    def receive(self, message, **kwargs):
        """Handle an incoming vehicle data message.

        Args:
            message (dict) - a new OpenXC vehicle data message

        Kwargs:
           data_remaining (bool) - if the originating data source can peek ahead
               in the data stream, this argument will True if there is more data
               available.
        """
        raise NotImplementedError("Don't use DataSink directly")



"""
C++ source code generator for the vehicle interface firmware.
"""
import os
import operator
import logging

from openxc.utils import find_file

LOG = logging.getLogger(__name__)


class CodeGenerator(object):
    """This class is used to build an implementation of the signals.h functions
    from one or more CAN message sets. The message sets must already be read
    into memory and parsed.
    """

    MAX_SIGNAL_STATES = 12
    GENERATED_CODE_VERSION = "7.x"

    def __init__(self, search_paths):
        self.search_paths = search_paths
        self.message_sets = []

    def build_source(self):
        lines = []
        lines.extend(self._build_header())
        lines.extend(self._build_extra_sources())
        lines.extend(self._build_message_sets())
        lines.extend(self._build_buses())
        lines.extend(self._build_messages())
        lines.extend(self._build_signal_states())
        lines.extend(self._build_signals())
        lines.extend(self._build_initializers())
        lines.extend(self._build_loop())
        lines.extend(self._build_commands())
        lines.extend(self._build_decoder())

        with open(os.path.join(os.path.dirname(__file__),
                'signals.cpp.footer')) as footer:
            lines.append("")
            lines.append(footer.read())

        return '\n'.join(lines)

    @property
    def sorted_message_sets(self):
        return sorted(self.message_sets, key=operator.attrgetter('name'))

    def _max_command_count(self):
        if len(self.message_sets) == 0:
            return 0

        return max(len(list(message_set.active_commands()))
                for message_set in self.message_sets)

    def _max_message_count(self):
        if len(self.message_sets) == 0:
            return 0
        return max(len(list(message_set.active_messages()))
                for message_set in self.message_sets)

    def _max_signal_count(self):
        if len(self.message_sets) == 0:
            return 0

        return max(len(list(message_set.enabled_signals()))
                for message_set in self.message_sets)

    def _build_header(self):
        with open(os.path.join(os.path.dirname(__file__),
                'signals.cpp.header')) as header:
            return [header.read().format(self.GENERATED_CODE_VERSION)]

    def _build_extra_sources(self):
        lines = []
        for i, message_set in enumerate(self.sorted_message_sets):
            for extra_source_filename in message_set.extra_sources:
                with open(find_file(extra_source_filename, self.search_paths)
                        ) as extra_source_file:
                    lines.append(extra_source_file.read())
        return lines

    def _build_message_set(self, index, message_set):
        LOG.info("Added message set '%s'" % message_set.name)
        return "    { %d, \"%s\", %d, %d, %d, %d }," % (index, message_set.name,
                len(list(message_set.valid_buses())),
                len(list(message_set.active_messages())),
                len(list(message_set.enabled_signals())),
                len(list(message_set.active_commands())))

    def _build_messages(self):
        lines = []
        lines.append("const int MAX_MESSAGE_COUNT = %d;" %
                self._max_message_count())
        lines.append("CanMessageDefinition CAN_MESSAGES[][MAX_MESSAGE_COUNT] = {")

        def block(message_set):
            lines = []
            for message_index, message in enumerate(message_set.all_messages()):
                if not message.active:
                    msg = "Skipping disabled message 0x%x" % message.id
                    if message.name is not None:
                        msg += " (%s)" % message.name
                    LOG.warning(msg)
                    continue
                LOG.info("Added message '%s'" % message.name)
                lines.append(" " * 8 + "%s" % message)
            return lines

        lines.extend(self._message_set_lister(block))

        lines.append("};")
        lines.append("")
        return lines

    def _build_message_sets(self):
        lines = []
        lines.append("const int MESSAGE_SET_COUNT = %d;" %
                len(self.message_sets))
        lines.append("CanMessageSet MESSAGE_SETS[MESSAGE_SET_COUNT] = {")
        for i, message_set in enumerate(self.sorted_message_sets):
            message_set.index = i
            lines.append(self._build_message_set(i, message_set))
        lines.append("};")
        lines.append("")
        return lines

    def _build_buses(self):
        lines = []
        lines.append("const int MAX_CAN_BUS_COUNT = 2;")
        lines.append("CanBus CAN_BUSES[][MAX_CAN_BUS_COUNT] = {")

        def block(message_set, **kwargs):
            lines = []
            for bus in message_set.valid_buses():
                lines.append(str(bus))
                lines.append("")
            return lines

        lines.extend(self._message_set_lister(block))
        lines.append("};")
        lines.append("")

        return lines

    def _build_signal_states(self):
        lines = []
        lines.append("const int MAX_SIGNAL_STATES = %d;" %
                self.MAX_SIGNAL_STATES)

        signals_with_states = 0
        for message_set in self.sorted_message_sets:
            for signal in message_set.enabled_signals():
                if len(signal.states) > 0:
                    signals_with_states += 1

        lines.append("const int MAX_SIGNALS_WITH_STATES_COUNT = %d;" %
                signals_with_states)

        lines.append("const CanSignalState SIGNAL_STATES[]"
                "[MAX_SIGNALS_WITH_STATES_COUNT][MAX_SIGNAL_STATES] = {")

        def block(message_set, **kwargs):
            states_index = 0
            lines = []
            for signal in message_set.enabled_signals():
                if len(signal.states) > 0:
                    line = " " * 8 + "{ "
                    for state_count, state in enumerate(signal.sorted_states):
                        if state_count >= self.MAX_SIGNAL_STATES:
                            LOG.warning("Ignoring anything beyond %d states for %s" %
                                    (self.MAX_SIGNAL_STATES,
                                        signal.generic_name))
                            break
                        line += "%s, " % state

                    # TODO this is dumb, but we have to fill in a bunch of const
                    # values for each signal state up to the array length
                    for _ in range(state_count + 1, self.MAX_SIGNAL_STATES):
                        line += "{ 0, NULL }, "

                    line += "},"
                    lines.append(line)
                    signal.states_index = states_index
                    states_index += 1
            # TODO this is dumb, but we have to fill in a bunch of const values
            for _ in range(len(list(message_set.enabled_signals())),
                    signals_with_states):
                line += "{ "
                for _ in range(self.MAX_SIGNAL_STATES):
                    line += "{ 0, NULL }, "
                line += " },"
            return lines

        # Since this is defined const, we can't include the interior array if
        # it's going to empty
        lines.extend(self._message_set_lister(block, always_include_braces=False))

        lines.append("};")
        lines.append("")

        return lines

    def _message_set_lister(self, block, indent=4, always_include_braces=True):
        lines = []
        whitespace = " " * indent
        for message_set in self.sorted_message_sets:
            interior = list(block(message_set))
            if len(interior) > 0:
                lines.append(whitespace + "{ // message set: %s" % message_set.name)
                lines.extend(interior)
                lines.append(whitespace + "},")
        return lines

    def _message_set_switcher(self, block, indent=4):
        lines = []
        whitespace = " " * indent
        lines.append(whitespace + "switch(getConfiguration()"
                "->messageSetIndex) {")
        for message_set in self.sorted_message_sets:
            lines.append(whitespace + "case %d: // message set: %s" % (
                    message_set.index, message_set.name))
            lines.extend(block(message_set))
            lines.append(whitespace * 2 + "break;")
            lines.append(whitespace + "}")
        return lines

    def _build_signals(self):
        lines = []
        lines.append("const int MAX_SIGNAL_COUNT = %d;" %
                self._max_signal_count())
        lines.append("CanSignal SIGNALS[][MAX_SIGNAL_COUNT] = {")

        def block(message_set):
            lines = []
            i = 1
            for signal in message_set.all_signals():
                if not signal.enabled:
                    LOG.warning("Skipping disabled signal '%s' (in 0x%x)" % (
                        signal.generic_name, signal.message.id))
                    continue
                signal.array_index = i - 1
                lines.append(" " * 8 + "%s" % signal)
                LOG.info("Added signal '%s'" % signal.generic_name)
                i += 1
            return lines

        lines.extend(self._message_set_lister(block))
        lines.append("};")
        lines.append("")

        return lines

    def _build_initializers(self):
        lines = []
        lines.append("void openxc::signals::initialize(openxc::diagnostics::DiagnosticsManager* diagnosticsManager) {")

        def block(message_set):
            init_lines = [" " * 8 + "%s();" % initializer
                for initializer in message_set.initializers]
            init_lines.extend([" " * 8 + "%s" % message
                for message in message_set.active_diagnostic_messages()])
            return init_lines
        lines.extend(self._message_set_switcher(block))

        lines.append("}")
        lines.append("")
        return lines

    def _build_loop(self):
        lines = []
        lines.append("void openxc::signals::loop() {")
        def block(message_set):
            return [" " * 8 + "%s();" % looper for looper in message_set.loopers]
        lines.extend(self._message_set_switcher(block))
        lines.append("}")
        lines.append("")
        return lines

    def _build_commands(self):
        lines = []
        lines.append("const int MAX_COMMAND_COUNT = %d;" %
                self._max_command_count())
        lines.append("CanCommand COMMANDS[][MAX_COMMAND_COUNT] = {")
        def block(message_set, **kwargs):
            for command in message_set.all_commands():
                if not command.enabled:
                    LOG.warning("Skipping disabled Command %s" % command.name)
                    continue
                LOG.info("Added command '%s'" % command.name)
                yield " " * 8 + "%s" % command
        lines.extend(self._message_set_lister(block))
        lines.append("};")
        lines.append("")

        return lines

    def _build_decoder(self):
        lines = []
        lines.append("void openxc::signals::decodeCanMessage("
                "Pipeline* pipeline, CanBus* bus, CanMessage* message) {")

        def block(message_set):
            lines = []
            lines.append(" " * 8 + "switch(bus->address) {")
            for bus in message_set.valid_buses():
                lines.append(" " * 8 + "case %s:" % bus.controller)
                lines.append(" " * 12 + "switch (message->id) {")
                for message in bus.active_messages():
                    if (len(list(message.active_signals())) > 0 or
                            len(list(message.handlers)) > 0):
                        lines.append(" " * 12 + "case 0x%x: // %s" % (message.id,
                                message.name))
                        for handler in message.handlers:
                            lines.append(" " * 16 + "%s(message, SIGNALS[%d], " % (
                                handler, message_set.index) +
                                    "getSignalCount(), pipeline);")
                        for signal in message.active_signals():
                            line = " " * 16
                            line += ("can::read::translateSignal("
                                        "&SIGNALS[%d][%d], message, " %
                                        (message_set.index, signal.array_index))
                            line += ("SIGNALS[%d], getSignalCount(), pipeline); // %s" % (
                                message_set.index, signal.name))
                            lines.append(line)
                        lines.append(" " * 16 + "break;")
                lines.append(" " * 12 + "}")
                lines.append(" " * 12 + "break;")
            lines.append(" " * 8 + "}")
            return lines

        lines.extend(self._message_set_switcher(block))

        lines.append("}")
        lines.append("")

        return lines



"""A Bluetooth data source."""
from __future__ import absolute_import

import logging

from openxc.controllers.base import Controller
from .socket import SocketDataSource
from .base import DataSourceError

LOG = logging.getLogger(__name__)

try:
    import bluetooth
except ImportError:
    LOG.debug("pybluez library not installed, can't use bluetooth interface")
    bluetooth = None


class BluetoothVehicleInterface(SocketDataSource, Controller):
    """A data source reading from a bluetooth device.
    """

    OPENXC_DEVICE_NAME_PREFIX = "OpenXC-VI-"

    def __init__(self, address=None, **kwargs):
        """Initialize a connection to the bluetooth device.

        Raises:
            DataSourceError if the bluetooth device cannot be opened.
        """
        super(BluetoothVehicleInterface, self).__init__(**kwargs)
        self.address = address

        if bluetooth is None:
            raise DataSourceError("pybluez library is not available")

        while self.address is None:
            self.scan_for_bluetooth_device()
        self.connect()

    def connect(self):
        # TODO push this to a background connecting thread so the constructor
        # can return
        port = 1
        connected = False
        while not connected:
            self.socket = bluetooth.BluetoothSocket(bluetooth.RFCOMM)
            try:
                self.socket.connect((self.address, port))
            except IOError as e:
                LOG.warn("Unable to connect to %s" % self.address, e)
            else:
                LOG.info("Opened bluetooth device at %s", port)
                connected = True

    def scan_for_bluetooth_device(self):
        nearby_devices = bluetooth.discover_devices()

        self.address = None
        device_name = None
        for address in nearby_devices:
            device_name = bluetooth.lookup_name(address)
            if (device_name is not None and
                    device_name.startswith(self.OPENXC_DEVICE_NAME_PREFIX)):
                self.address = address
                break

        if self.address is not None:
            LOG.info("Discovered OpenXC VI %s (%s)" % (device_name, self.address))
        else:
            LOG.info("No OpenXC VI devices discovered")



import operator
import math
from collections import defaultdict

import logging

LOG = logging.getLogger(__name__)


class Command(object):
    def __init__(self, name=None, handler=None, enabled=True, **kwargs):
        self.name = name
        self.handler = handler
        self.enabled = enabled

    def __str__(self):
        return "{ genericName: \"%s\", handler: %s }," % (self.name, self.handler)


class DiagnosticMessage(object):
    def __init__(self, message_set, enabled=True, **kwargs):
        self.message_set = message_set
        self.enabled = enabled
        self.id = kwargs['id']

        self.bus_name = kwargs['bus']
        if not isinstance(self.bus_name, unicode):
            raise ConfigurationError("Bus name must be name of bus in "
                    "config, was '%s'" % self.bus_name)

        self.bus = self.message_set.lookup_bus_index(self.bus_name)
        if self.bus is None:
            raise ConfigurationError("Unable to find bus with name '%s'"
                    % self.bus_name)

        self.name = kwargs.get('name', None)
        self.multiple_responses = kwargs.get('multiple_responses', False)
        self.decoder = kwargs.get('decoder', None)
        self.callback = kwargs.get('callback', None)
        self.mode = kwargs['mode']
        self.pid = kwargs.get('pid', 0)
        self.pid_length = kwargs.get('pid_length', 0)

        try:
            self.frequency = kwargs['frequency']
        except KeyError:
            raise ConfigurationError("Frequency must be specified for "
                "pre-configured diagnostic request (bus: "
                "%s, id: %d, mode: %d, pid: %d" % (self.bus_name,
                        self.id, self.mode, self.pid))

    def __str__(self):
        result = "{\n        DiagnosticRequest request = {arbitration_id: 0x%x, mode: 0x%x, has_pid: %s, pid: 0x%x, pid_length: %d};\n" % (
                self.id, self.mode, str(self.pid > 0).lower(), self.pid,
                self.pid_length)

        if self.name:
            name = "\"%s\"" % self.name
        else:
            name = "NULL"

        result += " " * 8 + "addRecurringRequest(diagnosticsManager, &getCanBuses()[%d], &request, %s, %s, %s, %s, %f);\n        }\n" % (
                self.message_set.lookup_bus_index(self.bus_name),
                name,
                str(self.multiple_responses).lower(),
                self.decoder or "NULL",
                self.callback or "NULL",
                self.frequency)
        return result


class Message(object):
    def __init__(self, bus_name=None, id=None, name=None,
            bit_numbering_inverted=None, handlers=None, enabled=True):
        self.bus_name = bus_name
        self.id = id
        self.name = name
        self.bit_numbering_inverted = bit_numbering_inverted
        self.handlers = handlers or []
        self.enabled = enabled
        self.max_frequency = None
        self.max_signal_frequency = 0
        self.force_send_changed = True
        self.force_send_changed_signals = False
        self.signals = defaultdict(Signal)

    @property
    def active(self):
        return self.enabled

    @property
    def id(self):
        return getattr(self, '_id', None)

    @id.setter
    def id(self, value):
        if value is not None:
            if not isinstance(value, int):
                value = int(value, 0)
            self._id = value

    def merge_message(self, data):
        self.bus_name = self.bus_name or data.get('bus', None)

        if getattr(self, 'message_set'):
            self.bus = self.message_set.lookup_bus(name=self.bus_name)
            if not self.bus.valid():
                self.enabled = False
                msg = ""
                if self.bus is None:
                    msg = "Bus '%s' is invalid, only %s are defined" % (
                            self.bus_name, list(self.message_set.valid_buses()))
                else:
                    msg = "Bus '%s' is disabled" % self.bus_name
                LOG.warning("%s - message 0x%x will be disabled" % (msg, self.id))

        self.id = self.id or data.get('id')
        self.name = self.name or data.get('name', None)
        self.bit_numbering_inverted = (self.bit_numbering_inverted or
                data.get('bit_numbering_inverted', None))

        self.max_frequency = data.get('max_frequency', self.max_frequency)
        if self.max_frequency is None:
            self.max_frequency = self.bus.max_message_frequency

        self.max_signal_frequency = data.get('max_signal_frequency',
                self.max_signal_frequency)
        self.force_send_changed = data.get('force_send_changed',
                self.force_send_changed)
        self.force_send_changed_signals = data.get('force_send_changed_signals',
                self.force_send_changed_signals)
        self.handlers.extend(data.get('handlers', []))
        if 'handler' in data:
            # Support deprecated single 'handler' field
            self.handlers.append(data.get('handler'))
            LOG.warning("The 'handler' attribute on the message " +
                    "%s is deprecated but will still work for " % self.name +
                    "now - the replacement is a 'handlers' array")
        if self.enabled is None:
            self.enabled = data.get('enabled', True)
        else:
            self.enabled = data.get('enabled', self.enabled)

        if 'signals' in data:
            self.merge_signals(data['signals'])

    def merge_signals(self, data):
        for signal_name, signal_data in data.items():
            states = []
            for name, raw_matches in signal_data.get('states', {}).items():
                for raw_match in raw_matches:
                    states.append(SignalState(raw_match, name))
            signal_data.pop('states', None)

            signal = self.signals[signal_name]
            signal.name = signal_name
            signal.message_set = self.message_set
            signal.message = self
            # TODO this will clobber existing states but I don't have a really
            # obvious clean solution to it at the moment
            signal.states = states
            signal.merge_signal(signal_data)

    def validate(self):
        if self.bus_name is None:
            LOG.warning("No default or explicit bus for message %s" % self.id)
            return False

        if self.bus_name not in self.message_set.buses:
            LOG.warning("Bus '%s' (from message 0x%x) is not defined" %
                    (self.bus_name, self.id))
            return False
        return True

    def sorted_signals(self):
        for signal in sorted(self.enabled_signals(),
                key=operator.attrgetter('generic_name')):
            yield signal

    def enabled_signals(self):
        for signal in self.signals.values():
            if signal.enabled:
                yield signal

    def active_signals(self):
        """An active signal is one that's enabled and not ignored, i.e. it
        should be translated.
        """
        for signal in self.sorted_signals():
            if not signal.ignore:
                yield signal

    def to_dict(self):
        return {"name": self.name,
                "signals": dict((key, value.to_dict())
                    for key, value in self.signals.items())}

    def __str__(self):
        bus_index = self.message_set.lookup_bus_index(self.bus_name)
        if bus_index is not None:
            id_format = "STANDARD";
            if self.id > 2047:
                id_format = "EXTENDED";
            return "{ bus: &CAN_BUSES[%d][%d], id: 0x%x, format: %s, frequencyClock: {%f}, forceSendChanged: %s}, // %s" % (
                    self.message_set.index, bus_index, self.id,
                    id_format,
                    self.max_frequency,
                    str(self.force_send_changed).lower(),
                    self.name)
        else:
            bus = self.message_set.lookup_bus(name=self.bus_name)
            msg = ""
            if bus is None:
                msg = "Bus '%s' is invalid, only %s are defined" % (
                        self.bus_name, list(self.message_set.valid_buses()))
            else:
                msg = "Bus '%s' is disabled" % self.bus_name
            LOG.warning("%s - message 0x%x will be disabled\n" % (msg, self.id))
        return ""

class CanBus(object):
    # Only works with 2 CAN buses since we are limited by 2 CAN controllers,
    # and we want to be a little careful that we always expect 0x101 to be
    # plugged into the CAN1 controller and 0x102 into CAN2.
    VALID_BUS_ADDRESSES = (1, 2)

    def __init__(self, name=None, speed=None, controller=None,
            default_max_message_frequency=None,
            max_message_frequency=None,
            default_raw_can_mode=None,
            raw_can_mode=None,
            raw_writable=False,
            loopback=False,
            **kwargs):
        self.name = name
        self.speed = speed
        self.messages = defaultdict(Message)
        self.controller = controller
        self.max_message_frequency = max_message_frequency
        if self.max_message_frequency is None:
            self.max_message_frequency = default_max_message_frequency
        self.raw_can_mode = raw_can_mode or default_raw_can_mode
        self.raw_writable = raw_writable
        self.loopback = loopback

    def valid(self):
        return self.controller in self.VALID_BUS_ADDRESSES

    def active_messages(self):
        for message in self.sorted_messages():
            if message.active:
                yield message

    def enabled_signals(self):
        for message in self.active_messages():
            for signal in message.enabled_signals():
                yield signal

    def sorted_messages(self):
        for message in sorted(self.messages.values(),
                key=operator.attrgetter('id')):
            yield message

    def get_or_create_message(self, message_id):
        return self.messages[message_id]

    def add_message(self, message):
        self.messages.append(message)

    @property
    def passthrough(self):
        return self.raw_can_mode != "off"

    @property
    def bypass_filters(self):
        return self.raw_can_mode == "unfiltered"

    def __str__(self):
        result = """        {{ speed: {bus_speed},
        address: {controller},
        maxMessageFrequency: {max_message_frequency},
        rawWritable: {raw_writable},
        passthroughCanMessages: {passthrough},
        bypassFilters: {bypass_filters},
        loopback: {loopback}
        }},"""
        return result.format(bus_speed=self.speed, controller=self.controller,
                max_message_frequency=self.max_message_frequency,
                raw_writable=str(self.raw_writable).lower(),
                passthrough=str(self.passthrough).lower(),
                bypass_filters=str(self.bypass_filters).lower(),
                loopback=str(self.loopback).lower())


class ConfigurationError(Exception):
    pass

class BitInversionError(Exception):
    pass


class Signal(object):
    def __init__(self, message_set=None, message=None, states=None, **kwargs):
        self.message_set = message_set
        self.message = message

        self.name = None
        self.generic_name = None
        self.bit_position = None
        self.bit_size = None
        self.decoder = None
        self.encoder = None
        self.factor = 1
        self.offset = 0
        self.min_value = 0.0
        self.max_value = 0.0
        self.max_frequency = None
        self.send_same = True
        self.force_send_changed = None
        self.writable = False
        self.enabled = True
        self.ignore = False
        self.bit_numbering_inverted = None
        self.states = states or []

        self.merge_signal(kwargs)

    def merge_signal(self, data):
        self.name = data.get('name', self.name)
        self.enabled = data.get('enabled', self.enabled)
        self.generic_name = data.get('generic_name', self.name)
        self.bit_position = data.get('bit_position', self.bit_position)
        self.bit_size = data.get('bit_size', self.bit_size)
        self.factor = data.get('factor', self.factor)
        self.offset = data.get('offset', self.offset)
        self.min_value = data.get('min_value', self.min_value)
        self.max_value = data.get('max_value', self.max_value)
        # Kind of nasty, but we want to avoid actually setting one of the
        # implicit handlers on the object (and then later on, assuming that it
        # was set explicitly)
        self.decoder = data.get('decoder', getattr(self, '_decoder', None))
        self.writable = data.get('writable', self.writable)
        self.encoder = data.get('encoder', self.encoder)
        self.send_same = data.get('send_same', self.send_same)
        self.force_send_changed = data.get('force_send_changed',
                self.force_send_changed)
        self.ignore = data.get('ignore', self.ignore)
        self.max_frequency = data.get('max_frequency', self.max_frequency)
        self.bit_numbering_inverted = data.get('bit_numbering_inverted')

        if 'send_frequency' in data:
            LOG.warning("The 'send_frequency' attribute in the signal " +
                    "%s is deprecated and has no effect " % self.generic_name +
                    " - see the replacement, max_frequency")

    @property
    def decoder(self):
        decoder = getattr(self, '_decoder', None)
        if decoder is None:
            if self.ignore:
                decoder = "ignoreDecoder"
            elif len(self.states) > 0:
                decoder = "stateDecoder"
        return decoder

    @decoder.setter
    def decoder(self, value):
        self._decoder = value

    @property
    def enabled(self):
        signal_enabled = getattr(self, '_enabled', True)
        if self.message is not None:
            return self.message.enabled and signal_enabled
        return signal_enabled

    @enabled.setter
    def enabled(self, value):
        self._enabled = value

    @property
    def max_frequency(self):
        max_freq = getattr(self, '_max_frequency', None)
        if max_freq is None and self.message is not None:
            max_freq = self.message.max_signal_frequency
        return max_freq

    @max_frequency.setter
    def max_frequency(self, value):
        self._max_frequency = value

    @property
    def force_send_changed(self):
        force_send = getattr(self, '_force_send_changed', None)
        if force_send is None and self.message is not None:
            force_send = self.message.force_send_changed_signals
        return force_send

    @force_send_changed.setter
    def force_send_changed(self, value):
        self._force_send_changed = value

    @property
    def sorted_states(self):
        return sorted(self.states, key=operator.attrgetter('value'))

    def to_dict(self):
        return {"generic_name": self.generic_name,
                "bit_position": self.bit_position,
                "bit_size": self.bit_size,
                "factor": self.factor,
                "offset": self.offset,
                "min_value": self.min_value,
                "max_value": self.max_value}

    def translate(self, can_data):
        translated_value = int(can_data, 16)

        # Trim off everything to the right
        translated_value >>= 64 - self.bit_position - self.bit_size
        mask = int(math.pow(2, self.bit_size)) - 1
        translated_value &= mask
        translated_value *= self.factor
        translated_value += self.offset
        return translated_value

    def validate(self):
        if self.send_same is False and self.max_frequency > 0:
            LOG.warning("Signal %s combines send_same and " % self.generic_name +
                    "max_frequency - this is not recommended")
        if self.bit_position == None or self.bit_size == None:
            LOG.error("%s (generic name: %s) is incomplete\n" %
                    (self.name, self.generic_name))
            return False
        return True

    @property
    def bit_position(self):
        if (getattr(self, '_bit_position', None) is not None and
                self.bit_numbering_inverted):
            return self._invert_bit_index(self._bit_position, self.bit_size)
        else:
            return self._bit_position

    @bit_position.setter
    def bit_position(self, value):
        self._bit_position = value

    @property
    def bit_numbering_inverted(self):
        if getattr(self, '_bit_numbering_inverted', None) is None:
            return getattr(self.message, 'bit_numbering_inverted', False)
        return self._bit_numbering_inverted

    @bit_numbering_inverted.setter
    def bit_numbering_inverted(self, value):
        if value != None:
            self._bit_numbering_inverted = value

    @classmethod
    def _invert_bit_index(cls, bit_index, length):
        (b, r) = divmod(bit_index, 8)
        end = (8 * b) + (7 - r)
        inverted_index = end - length + 1
        if inverted_index < 0:
            raise BitInversionError("Bit index %d with length %d cannot be " %
                        (bit_index, length) +
                    "inverted - you probably need to disable the "
                    "'bit_numbering_inverted' flag in your JSON mapping")
        return inverted_index

    def __str__(self):
        result =  ("{message: &CAN_MESSAGES[%d][%d], genericName: \"%s\", bitPosition: %s, bitSize: %d, factor: %f, offset: %f, minValue: %f, maxValue: %f, frequencyClock: {%f}, sendSame: %s, forceSendChanged: %s, " % (
                self.message_set.index,
                self.message_set.lookup_message_index(self.message),
                self.generic_name, self.bit_position, self.bit_size,
                self.factor, self.offset, self.min_value, self.max_value,
                self.max_frequency, str(self.send_same).lower(),
                str(self.force_send_changed).lower()))
        if len(self.states) > 0:
            result += "states: SIGNAL_STATES[%d][%d], stateCount: %d" % (self.message_set.index,
                    self.states_index, len(self.states))
        else:
            result += "states: NULL, stateCount: 0"
        result += ", writable: %s, decoder: %s, encoder: %s" % (str(self.writable).lower(),
                self.decoder or "NULL",
                self.encoder or "NULL")
        result += "}, // %s" % self.name
        return result


class SignalState(object):
    def __init__(self, value, name):
        self.value = value
        self.name = name

    def __str__(self):
        return "{value: %d, name: \"%s\"}" % (self.value, self.name)



# Generated by the protocol buffer compiler.  DO NOT EDIT!

from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)



DESCRIPTOR = descriptor.FileDescriptor(
  name='openxc.proto',
  package='openxc',
  serialized_pb='\n\x0copenxc.proto\x12\x06openxc\"\x9b\x03\n\x0eVehicleMessage\x12)\n\x04type\x18\x01 \x01(\x0e\x32\x1b.openxc.VehicleMessage.Type\x12\'\n\x0b\x63\x61n_message\x18\x02 \x01(\x0b\x32\x12.openxc.CanMessage\x12-\n\x0esimple_message\x18\x03 \x01(\x0b\x32\x15.openxc.SimpleMessage\x12\x37\n\x13\x64iagnostic_response\x18\x04 \x01(\x0b\x32\x1a.openxc.DiagnosticResponse\x12/\n\x0f\x63ontrol_command\x18\x05 \x01(\x0b\x32\x16.openxc.ControlCommand\x12\x31\n\x10\x63ommand_response\x18\x06 \x01(\x0b\x32\x17.openxc.CommandResponse\x12\x11\n\ttimestamp\x18\x07 \x01(\x04\"V\n\x04Type\x12\x07\n\x03\x43\x41N\x10\x01\x12\n\n\x06SIMPLE\x10\x02\x12\x0e\n\nDIAGNOSTIC\x10\x03\x12\x13\n\x0f\x43ONTROL_COMMAND\x10\x04\x12\x14\n\x10\x43OMMAND_RESPONSE\x10\x05\"\x94\x01\n\nCanMessage\x12\x0b\n\x03\x62us\x18\x01 \x01(\x05\x12\n\n\x02id\x18\x02 \x01(\r\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\x12\x34\n\x0c\x66rame_format\x18\x04 \x01(\x0e\x32\x1e.openxc.CanMessage.FrameFormat\")\n\x0b\x46rameFormat\x12\x0c\n\x08STANDARD\x10\x01\x12\x0c\n\x08\x45XTENDED\x10\x02\"\x89\x06\n\x0e\x43ontrolCommand\x12)\n\x04type\x18\x01 \x01(\x0e\x32\x1b.openxc.ControlCommand.Type\x12<\n\x12\x64iagnostic_request\x18\x02 \x01(\x0b\x32 .openxc.DiagnosticControlCommand\x12G\n\x18passthrough_mode_request\x18\x03 \x01(\x0b\x32%.openxc.PassthroughModeControlCommand\x12O\n acceptance_filter_bypass_command\x18\x04 \x01(\x0b\x32%.openxc.AcceptanceFilterBypassCommand\x12<\n\x16payload_format_command\x18\x05 \x01(\x0b\x32\x1c.openxc.PayloadFormatCommand\x12O\n predefined_obd2_requests_command\x18\x06 \x01(\x0b\x32%.openxc.PredefinedObd2RequestsCommand\x12\x46\n\x1bmodem_configuration_command\x18\x07 \x01(\x0b\x32!.openxc.ModemConfigurationCommand\x12\x42\n\x19rtc_configuration_command\x18\x08 \x01(\x0b\x32\x1f.openxc.RTCConfigurationCommand\"\xd8\x01\n\x04Type\x12\x0b\n\x07VERSION\x10\x01\x12\r\n\tDEVICE_ID\x10\x02\x12\x0e\n\nDIAGNOSTIC\x10\x03\x12\x0f\n\x0bPASSTHROUGH\x10\x04\x12\x1c\n\x18\x41\x43\x43\x45PTANCE_FILTER_BYPASS\x10\x05\x12\x12\n\x0ePAYLOAD_FORMAT\x10\x06\x12\x1c\n\x18PREDEFINED_OBD2_REQUESTS\x10\x07\x12\x17\n\x13MODEM_CONFIGURATION\x10\x08\x12\x15\n\x11RTC_CONFIGURATION\x10\t\x12\x13\n\x0fSD_MOUNT_STATUS\x10\n\"\x9e\x01\n\x18\x44iagnosticControlCommand\x12*\n\x07request\x18\x01 \x01(\x0b\x32\x19.openxc.DiagnosticRequest\x12\x37\n\x06\x61\x63tion\x18\x02 \x01(\x0e\x32\'.openxc.DiagnosticControlCommand.Action\"\x1d\n\x06\x41\x63tion\x12\x07\n\x03\x41\x44\x44\x10\x01\x12\n\n\x06\x43\x41NCEL\x10\x02\"=\n\x1dPassthroughModeControlCommand\x12\x0b\n\x03\x62us\x18\x01 \x01(\x05\x12\x0f\n\x07\x65nabled\x18\x02 \x01(\x08\"<\n\x1d\x41\x63\x63\x65ptanceFilterBypassCommand\x12\x0b\n\x03\x62us\x18\x01 \x01(\x05\x12\x0e\n\x06\x62ypass\x18\x02 \x01(\x08\"\x8c\x01\n\x14PayloadFormatCommand\x12:\n\x06\x66ormat\x18\x01 \x01(\x0e\x32*.openxc.PayloadFormatCommand.PayloadFormat\"8\n\rPayloadFormat\x12\x08\n\x04JSON\x10\x01\x12\x0c\n\x08PROTOBUF\x10\x02\x12\x0f\n\x0bMESSAGEPACK\x10\x03\"0\n\x1dPredefinedObd2RequestsCommand\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\"\xd1\x03\n\x17NetworkOperatorSettings\x12\x18\n\x10\x61llowDataRoaming\x18\x01 \x01(\x08\x12N\n\x12operatorSelectMode\x18\x02 \x01(\x0e\x32\x32.openxc.NetworkOperatorSettings.OperatorSelectMode\x12L\n\x11networkDescriptor\x18\x03 \x01(\x0b\x32\x31.openxc.NetworkOperatorSettings.NetworkDescriptor\x1a\x98\x01\n\x11NetworkDescriptor\x12\x0c\n\x04PLMN\x18\x01 \x01(\r\x12R\n\x0bnetworkType\x18\x02 \x01(\x0e\x32=.openxc.NetworkOperatorSettings.NetworkDescriptor.NetworkType\"!\n\x0bNetworkType\x12\x07\n\x03GSM\x10\x00\x12\t\n\x05UTRAN\x10\x02\"c\n\x12OperatorSelectMode\x12\r\n\tAUTOMATIC\x10\x00\x12\n\n\x06MANUAL\x10\x01\x12\x0e\n\nDEREGISTER\x10\x02\x12\x0c\n\x08SET_ONLY\x10\x03\x12\x14\n\x10MANUAL_AUTOMATIC\x10\x04\"\"\n\x13NetworkDataSettings\x12\x0b\n\x03\x41PN\x18\x01 \x01(\t\"3\n\x15ServerConnectSettings\x12\x0c\n\x04host\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\r\"\xd5\x01\n\x19ModemConfigurationCommand\x12@\n\x17networkOperatorSettings\x18\x01 \x01(\x0b\x32\x1f.openxc.NetworkOperatorSettings\x12\x38\n\x13networkDataSettings\x18\x02 \x01(\x0b\x32\x1b.openxc.NetworkDataSettings\x12<\n\x15serverConnectSettings\x18\x03 \x01(\x0b\x32\x1d.openxc.ServerConnectSettings\",\n\x17RTCConfigurationCommand\x12\x11\n\tunix_time\x18\x01 \x01(\r\"]\n\x0f\x43ommandResponse\x12)\n\x04type\x18\x01 \x01(\x0e\x32\x1b.openxc.ControlCommand.Type\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\x08\"\xfd\x01\n\x11\x44iagnosticRequest\x12\x0b\n\x03\x62us\x18\x01 \x01(\x05\x12\x12\n\nmessage_id\x18\x02 \x01(\r\x12\x0c\n\x04mode\x18\x03 \x01(\r\x12\x0b\n\x03pid\x18\x04 \x01(\r\x12\x0f\n\x07payload\x18\x05 \x01(\x0c\x12\x1a\n\x12multiple_responses\x18\x06 \x01(\x08\x12\x11\n\tfrequency\x18\x07 \x01(\x01\x12\x0c\n\x04name\x18\x08 \x01(\t\x12;\n\x0c\x64\x65\x63oded_type\x18\t \x01(\x0e\x32%.openxc.DiagnosticRequest.DecodedType\"!\n\x0b\x44\x65\x63odedType\x12\x08\n\x04NONE\x10\x01\x12\x08\n\x04OBD2\x10\x02\"\xa1\x01\n\x12\x44iagnosticResponse\x12\x0b\n\x03\x62us\x18\x01 \x01(\x05\x12\x12\n\nmessage_id\x18\x02 \x01(\r\x12\x0c\n\x04mode\x18\x03 \x01(\r\x12\x0b\n\x03pid\x18\x04 \x01(\r\x12\x0f\n\x07success\x18\x05 \x01(\x08\x12\x1e\n\x16negative_response_code\x18\x06 \x01(\r\x12\x0f\n\x07payload\x18\x07 \x01(\x0c\x12\r\n\x05value\x18\x08 \x01(\x01\"\xa2\x01\n\x0c\x44ynamicField\x12\'\n\x04type\x18\x01 \x01(\x0e\x32\x19.openxc.DynamicField.Type\x12\x14\n\x0cstring_value\x18\x02 \x01(\t\x12\x15\n\rnumeric_value\x18\x03 \x01(\x01\x12\x15\n\rboolean_value\x18\x04 \x01(\x08\"%\n\x04Type\x12\n\n\x06STRING\x10\x01\x12\x07\n\x03NUM\x10\x02\x12\x08\n\x04\x42OOL\x10\x03\"g\n\rSimpleMessage\x12\x0c\n\x04name\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.openxc.DynamicField\x12#\n\x05\x65vent\x18\x03 \x01(\x0b\x32\x14.openxc.DynamicFieldB\x1c\n\ncom.openxcB\x0e\x42inaryMessages')



_VEHICLEMESSAGE_TYPE = descriptor.EnumDescriptor(
  name='Type',
  full_name='openxc.VehicleMessage.Type',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='CAN', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='SIMPLE', index=1, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='DIAGNOSTIC', index=2, number=3,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='CONTROL_COMMAND', index=3, number=4,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='COMMAND_RESPONSE', index=4, number=5,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=350,
  serialized_end=436,
)

_CANMESSAGE_FRAMEFORMAT = descriptor.EnumDescriptor(
  name='FrameFormat',
  full_name='openxc.CanMessage.FrameFormat',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='STANDARD', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='EXTENDED', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=546,
  serialized_end=587,
)

_CONTROLCOMMAND_TYPE = descriptor.EnumDescriptor(
  name='Type',
  full_name='openxc.ControlCommand.Type',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='VERSION', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='DEVICE_ID', index=1, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='DIAGNOSTIC', index=2, number=3,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='PASSTHROUGH', index=3, number=4,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ACCEPTANCE_FILTER_BYPASS', index=4, number=5,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='PAYLOAD_FORMAT', index=5, number=6,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='PREDEFINED_OBD2_REQUESTS', index=6, number=7,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='MODEM_CONFIGURATION', index=7, number=8,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='RTC_CONFIGURATION', index=8, number=9,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='SD_MOUNT_STATUS', index=9, number=10,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1151,
  serialized_end=1367,
)

_DIAGNOSTICCONTROLCOMMAND_ACTION = descriptor.EnumDescriptor(
  name='Action',
  full_name='openxc.DiagnosticControlCommand.Action',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='ADD', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='CANCEL', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1499,
  serialized_end=1528,
)

_PAYLOADFORMATCOMMAND_PAYLOADFORMAT = descriptor.EnumDescriptor(
  name='PayloadFormat',
  full_name='openxc.PayloadFormatCommand.PayloadFormat',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='JSON', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='PROTOBUF', index=1, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='MESSAGEPACK', index=2, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1740,
  serialized_end=1796,
)

_NETWORKOPERATORSETTINGS_NETWORKDESCRIPTOR_NETWORKTYPE = descriptor.EnumDescriptor(
  name='NetworkType',
  full_name='openxc.NetworkOperatorSettings.NetworkDescriptor.NetworkType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='GSM', index=0, number=0,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='UTRAN', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2180,
  serialized_end=2213,
)

_NETWORKOPERATORSETTINGS_OPERATORSELECTMODE = descriptor.EnumDescriptor(
  name='OperatorSelectMode',
  full_name='openxc.NetworkOperatorSettings.OperatorSelectMode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='AUTOMATIC', index=0, number=0,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='MANUAL', index=1, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='DEREGISTER', index=2, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='SET_ONLY', index=3, number=3,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='MANUAL_AUTOMATIC', index=4, number=4,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2215,
  serialized_end=2314,
)

_DIAGNOSTICREQUEST_DECODEDTYPE = descriptor.EnumDescriptor(
  name='DecodedType',
  full_name='openxc.DiagnosticRequest.DecodedType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='NONE', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='OBD2', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2983,
  serialized_end=3016,
)

_DYNAMICFIELD_TYPE = descriptor.EnumDescriptor(
  name='Type',
  full_name='openxc.DynamicField.Type',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='STRING', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='NUM', index=1, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='BOOL', index=2, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=3308,
  serialized_end=3345,
)


_VEHICLEMESSAGE = descriptor.Descriptor(
  name='VehicleMessage',
  full_name='openxc.VehicleMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='type', full_name='openxc.VehicleMessage.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='can_message', full_name='openxc.VehicleMessage.can_message', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='simple_message', full_name='openxc.VehicleMessage.simple_message', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='diagnostic_response', full_name='openxc.VehicleMessage.diagnostic_response', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='control_command', full_name='openxc.VehicleMessage.control_command', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='command_response', full_name='openxc.VehicleMessage.command_response', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='timestamp', full_name='openxc.VehicleMessage.timestamp', index=6,
      number=7, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _VEHICLEMESSAGE_TYPE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=25,
  serialized_end=436,
)


_CANMESSAGE = descriptor.Descriptor(
  name='CanMessage',
  full_name='openxc.CanMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='bus', full_name='openxc.CanMessage.bus', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='id', full_name='openxc.CanMessage.id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='data', full_name='openxc.CanMessage.data', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='frame_format', full_name='openxc.CanMessage.frame_format', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _CANMESSAGE_FRAMEFORMAT,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=439,
  serialized_end=587,
)


_CONTROLCOMMAND = descriptor.Descriptor(
  name='ControlCommand',
  full_name='openxc.ControlCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='type', full_name='openxc.ControlCommand.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='diagnostic_request', full_name='openxc.ControlCommand.diagnostic_request', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='passthrough_mode_request', full_name='openxc.ControlCommand.passthrough_mode_request', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='acceptance_filter_bypass_command', full_name='openxc.ControlCommand.acceptance_filter_bypass_command', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='payload_format_command', full_name='openxc.ControlCommand.payload_format_command', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='predefined_obd2_requests_command', full_name='openxc.ControlCommand.predefined_obd2_requests_command', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='modem_configuration_command', full_name='openxc.ControlCommand.modem_configuration_command', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='rtc_configuration_command', full_name='openxc.ControlCommand.rtc_configuration_command', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _CONTROLCOMMAND_TYPE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=590,
  serialized_end=1367,
)


_DIAGNOSTICCONTROLCOMMAND = descriptor.Descriptor(
  name='DiagnosticControlCommand',
  full_name='openxc.DiagnosticControlCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='request', full_name='openxc.DiagnosticControlCommand.request', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='action', full_name='openxc.DiagnosticControlCommand.action', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DIAGNOSTICCONTROLCOMMAND_ACTION,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1370,
  serialized_end=1528,
)


_PASSTHROUGHMODECONTROLCOMMAND = descriptor.Descriptor(
  name='PassthroughModeControlCommand',
  full_name='openxc.PassthroughModeControlCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='bus', full_name='openxc.PassthroughModeControlCommand.bus', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='enabled', full_name='openxc.PassthroughModeControlCommand.enabled', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1530,
  serialized_end=1591,
)


_ACCEPTANCEFILTERBYPASSCOMMAND = descriptor.Descriptor(
  name='AcceptanceFilterBypassCommand',
  full_name='openxc.AcceptanceFilterBypassCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='bus', full_name='openxc.AcceptanceFilterBypassCommand.bus', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='bypass', full_name='openxc.AcceptanceFilterBypassCommand.bypass', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1593,
  serialized_end=1653,
)


_PAYLOADFORMATCOMMAND = descriptor.Descriptor(
  name='PayloadFormatCommand',
  full_name='openxc.PayloadFormatCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='format', full_name='openxc.PayloadFormatCommand.format', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _PAYLOADFORMATCOMMAND_PAYLOADFORMAT,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1656,
  serialized_end=1796,
)


_PREDEFINEDOBD2REQUESTSCOMMAND = descriptor.Descriptor(
  name='PredefinedObd2RequestsCommand',
  full_name='openxc.PredefinedObd2RequestsCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='enabled', full_name='openxc.PredefinedObd2RequestsCommand.enabled', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1798,
  serialized_end=1846,
)


_NETWORKOPERATORSETTINGS_NETWORKDESCRIPTOR = descriptor.Descriptor(
  name='NetworkDescriptor',
  full_name='openxc.NetworkOperatorSettings.NetworkDescriptor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='PLMN', full_name='openxc.NetworkOperatorSettings.NetworkDescriptor.PLMN', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='networkType', full_name='openxc.NetworkOperatorSettings.NetworkDescriptor.networkType', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _NETWORKOPERATORSETTINGS_NETWORKDESCRIPTOR_NETWORKTYPE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2061,
  serialized_end=2213,
)

_NETWORKOPERATORSETTINGS = descriptor.Descriptor(
  name='NetworkOperatorSettings',
  full_name='openxc.NetworkOperatorSettings',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='allowDataRoaming', full_name='openxc.NetworkOperatorSettings.allowDataRoaming', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='operatorSelectMode', full_name='openxc.NetworkOperatorSettings.operatorSelectMode', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='networkDescriptor', full_name='openxc.NetworkOperatorSettings.networkDescriptor', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_NETWORKOPERATORSETTINGS_NETWORKDESCRIPTOR, ],
  enum_types=[
    _NETWORKOPERATORSETTINGS_OPERATORSELECTMODE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1849,
  serialized_end=2314,
)


_NETWORKDATASETTINGS = descriptor.Descriptor(
  name='NetworkDataSettings',
  full_name='openxc.NetworkDataSettings',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='APN', full_name='openxc.NetworkDataSettings.APN', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2316,
  serialized_end=2350,
)


_SERVERCONNECTSETTINGS = descriptor.Descriptor(
  name='ServerConnectSettings',
  full_name='openxc.ServerConnectSettings',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='host', full_name='openxc.ServerConnectSettings.host', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='port', full_name='openxc.ServerConnectSettings.port', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2352,
  serialized_end=2403,
)


_MODEMCONFIGURATIONCOMMAND = descriptor.Descriptor(
  name='ModemConfigurationCommand',
  full_name='openxc.ModemConfigurationCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='networkOperatorSettings', full_name='openxc.ModemConfigurationCommand.networkOperatorSettings', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='networkDataSettings', full_name='openxc.ModemConfigurationCommand.networkDataSettings', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='serverConnectSettings', full_name='openxc.ModemConfigurationCommand.serverConnectSettings', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2406,
  serialized_end=2619,
)


_RTCCONFIGURATIONCOMMAND = descriptor.Descriptor(
  name='RTCConfigurationCommand',
  full_name='openxc.RTCConfigurationCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='unix_time', full_name='openxc.RTCConfigurationCommand.unix_time', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2621,
  serialized_end=2665,
)


_COMMANDRESPONSE = descriptor.Descriptor(
  name='CommandResponse',
  full_name='openxc.CommandResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='type', full_name='openxc.CommandResponse.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='message', full_name='openxc.CommandResponse.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='status', full_name='openxc.CommandResponse.status', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2667,
  serialized_end=2760,
)


_DIAGNOSTICREQUEST = descriptor.Descriptor(
  name='DiagnosticRequest',
  full_name='openxc.DiagnosticRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='bus', full_name='openxc.DiagnosticRequest.bus', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='message_id', full_name='openxc.DiagnosticRequest.message_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='mode', full_name='openxc.DiagnosticRequest.mode', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='pid', full_name='openxc.DiagnosticRequest.pid', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='payload', full_name='openxc.DiagnosticRequest.payload', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='multiple_responses', full_name='openxc.DiagnosticRequest.multiple_responses', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='frequency', full_name='openxc.DiagnosticRequest.frequency', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='name', full_name='openxc.DiagnosticRequest.name', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='decoded_type', full_name='openxc.DiagnosticRequest.decoded_type', index=8,
      number=9, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DIAGNOSTICREQUEST_DECODEDTYPE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2763,
  serialized_end=3016,
)


_DIAGNOSTICRESPONSE = descriptor.Descriptor(
  name='DiagnosticResponse',
  full_name='openxc.DiagnosticResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='bus', full_name='openxc.DiagnosticResponse.bus', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='message_id', full_name='openxc.DiagnosticResponse.message_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='mode', full_name='openxc.DiagnosticResponse.mode', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='pid', full_name='openxc.DiagnosticResponse.pid', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='success', full_name='openxc.DiagnosticResponse.success', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='negative_response_code', full_name='openxc.DiagnosticResponse.negative_response_code', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='payload', full_name='openxc.DiagnosticResponse.payload', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='value', full_name='openxc.DiagnosticResponse.value', index=7,
      number=8, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=3019,
  serialized_end=3180,
)


_DYNAMICFIELD = descriptor.Descriptor(
  name='DynamicField',
  full_name='openxc.DynamicField',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='type', full_name='openxc.DynamicField.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='string_value', full_name='openxc.DynamicField.string_value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='numeric_value', full_name='openxc.DynamicField.numeric_value', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='boolean_value', full_name='openxc.DynamicField.boolean_value', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DYNAMICFIELD_TYPE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=3183,
  serialized_end=3345,
)


_SIMPLEMESSAGE = descriptor.Descriptor(
  name='SimpleMessage',
  full_name='openxc.SimpleMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='name', full_name='openxc.SimpleMessage.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='value', full_name='openxc.SimpleMessage.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='event', full_name='openxc.SimpleMessage.event', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=3347,
  serialized_end=3450,
)

_VEHICLEMESSAGE.fields_by_name['type'].enum_type = _VEHICLEMESSAGE_TYPE
_VEHICLEMESSAGE.fields_by_name['can_message'].message_type = _CANMESSAGE
_VEHICLEMESSAGE.fields_by_name['simple_message'].message_type = _SIMPLEMESSAGE
_VEHICLEMESSAGE.fields_by_name['diagnostic_response'].message_type = _DIAGNOSTICRESPONSE
_VEHICLEMESSAGE.fields_by_name['control_command'].message_type = _CONTROLCOMMAND
_VEHICLEMESSAGE.fields_by_name['command_response'].message_type = _COMMANDRESPONSE
_VEHICLEMESSAGE_TYPE.containing_type = _VEHICLEMESSAGE;
_CANMESSAGE.fields_by_name['frame_format'].enum_type = _CANMESSAGE_FRAMEFORMAT
_CANMESSAGE_FRAMEFORMAT.containing_type = _CANMESSAGE;
_CONTROLCOMMAND.fields_by_name['type'].enum_type = _CONTROLCOMMAND_TYPE
_CONTROLCOMMAND.fields_by_name['diagnostic_request'].message_type = _DIAGNOSTICCONTROLCOMMAND
_CONTROLCOMMAND.fields_by_name['passthrough_mode_request'].message_type = _PASSTHROUGHMODECONTROLCOMMAND
_CONTROLCOMMAND.fields_by_name['acceptance_filter_bypass_command'].message_type = _ACCEPTANCEFILTERBYPASSCOMMAND
_CONTROLCOMMAND.fields_by_name['payload_format_command'].message_type = _PAYLOADFORMATCOMMAND
_CONTROLCOMMAND.fields_by_name['predefined_obd2_requests_command'].message_type = _PREDEFINEDOBD2REQUESTSCOMMAND
_CONTROLCOMMAND.fields_by_name['modem_configuration_command'].message_type = _MODEMCONFIGURATIONCOMMAND
_CONTROLCOMMAND.fields_by_name['rtc_configuration_command'].message_type = _RTCCONFIGURATIONCOMMAND
_CONTROLCOMMAND_TYPE.containing_type = _CONTROLCOMMAND;
_DIAGNOSTICCONTROLCOMMAND.fields_by_name['request'].message_type = _DIAGNOSTICREQUEST
_DIAGNOSTICCONTROLCOMMAND.fields_by_name['action'].enum_type = _DIAGNOSTICCONTROLCOMMAND_ACTION
_DIAGNOSTICCONTROLCOMMAND_ACTION.containing_type = _DIAGNOSTICCONTROLCOMMAND;
_PAYLOADFORMATCOMMAND.fields_by_name['format'].enum_type = _PAYLOADFORMATCOMMAND_PAYLOADFORMAT
_PAYLOADFORMATCOMMAND_PAYLOADFORMAT.containing_type = _PAYLOADFORMATCOMMAND;
_NETWORKOPERATORSETTINGS_NETWORKDESCRIPTOR.fields_by_name['networkType'].enum_type = _NETWORKOPERATORSETTINGS_NETWORKDESCRIPTOR_NETWORKTYPE
_NETWORKOPERATORSETTINGS_NETWORKDESCRIPTOR.containing_type = _NETWORKOPERATORSETTINGS;
_NETWORKOPERATORSETTINGS_NETWORKDESCRIPTOR_NETWORKTYPE.containing_type = _NETWORKOPERATORSETTINGS_NETWORKDESCRIPTOR;
_NETWORKOPERATORSETTINGS.fields_by_name['operatorSelectMode'].enum_type = _NETWORKOPERATORSETTINGS_OPERATORSELECTMODE
_NETWORKOPERATORSETTINGS.fields_by_name['networkDescriptor'].message_type = _NETWORKOPERATORSETTINGS_NETWORKDESCRIPTOR
_NETWORKOPERATORSETTINGS_OPERATORSELECTMODE.containing_type = _NETWORKOPERATORSETTINGS;
_MODEMCONFIGURATIONCOMMAND.fields_by_name['networkOperatorSettings'].message_type = _NETWORKOPERATORSETTINGS
_MODEMCONFIGURATIONCOMMAND.fields_by_name['networkDataSettings'].message_type = _NETWORKDATASETTINGS
_MODEMCONFIGURATIONCOMMAND.fields_by_name['serverConnectSettings'].message_type = _SERVERCONNECTSETTINGS
_COMMANDRESPONSE.fields_by_name['type'].enum_type = _CONTROLCOMMAND_TYPE
_DIAGNOSTICREQUEST.fields_by_name['decoded_type'].enum_type = _DIAGNOSTICREQUEST_DECODEDTYPE
_DIAGNOSTICREQUEST_DECODEDTYPE.containing_type = _DIAGNOSTICREQUEST;
_DYNAMICFIELD.fields_by_name['type'].enum_type = _DYNAMICFIELD_TYPE
_DYNAMICFIELD_TYPE.containing_type = _DYNAMICFIELD;
_SIMPLEMESSAGE.fields_by_name['value'].message_type = _DYNAMICFIELD
_SIMPLEMESSAGE.fields_by_name['event'].message_type = _DYNAMICFIELD
DESCRIPTOR.message_types_by_name['VehicleMessage'] = _VEHICLEMESSAGE
DESCRIPTOR.message_types_by_name['CanMessage'] = _CANMESSAGE
DESCRIPTOR.message_types_by_name['ControlCommand'] = _CONTROLCOMMAND
DESCRIPTOR.message_types_by_name['DiagnosticControlCommand'] = _DIAGNOSTICCONTROLCOMMAND
DESCRIPTOR.message_types_by_name['PassthroughModeControlCommand'] = _PASSTHROUGHMODECONTROLCOMMAND
DESCRIPTOR.message_types_by_name['AcceptanceFilterBypassCommand'] = _ACCEPTANCEFILTERBYPASSCOMMAND
DESCRIPTOR.message_types_by_name['PayloadFormatCommand'] = _PAYLOADFORMATCOMMAND
DESCRIPTOR.message_types_by_name['PredefinedObd2RequestsCommand'] = _PREDEFINEDOBD2REQUESTSCOMMAND
DESCRIPTOR.message_types_by_name['NetworkOperatorSettings'] = _NETWORKOPERATORSETTINGS
DESCRIPTOR.message_types_by_name['NetworkDataSettings'] = _NETWORKDATASETTINGS
DESCRIPTOR.message_types_by_name['ServerConnectSettings'] = _SERVERCONNECTSETTINGS
DESCRIPTOR.message_types_by_name['ModemConfigurationCommand'] = _MODEMCONFIGURATIONCOMMAND
DESCRIPTOR.message_types_by_name['RTCConfigurationCommand'] = _RTCCONFIGURATIONCOMMAND
DESCRIPTOR.message_types_by_name['CommandResponse'] = _COMMANDRESPONSE
DESCRIPTOR.message_types_by_name['DiagnosticRequest'] = _DIAGNOSTICREQUEST
DESCRIPTOR.message_types_by_name['DiagnosticResponse'] = _DIAGNOSTICRESPONSE
DESCRIPTOR.message_types_by_name['DynamicField'] = _DYNAMICFIELD
DESCRIPTOR.message_types_by_name['SimpleMessage'] = _SIMPLEMESSAGE

class VehicleMessage(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _VEHICLEMESSAGE
  
  # @@protoc_insertion_point(class_scope:openxc.VehicleMessage)

class CanMessage(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _CANMESSAGE
  
  # @@protoc_insertion_point(class_scope:openxc.CanMessage)

class ControlCommand(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _CONTROLCOMMAND
  
  # @@protoc_insertion_point(class_scope:openxc.ControlCommand)

class DiagnosticControlCommand(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _DIAGNOSTICCONTROLCOMMAND
  
  # @@protoc_insertion_point(class_scope:openxc.DiagnosticControlCommand)

class PassthroughModeControlCommand(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _PASSTHROUGHMODECONTROLCOMMAND
  
  # @@protoc_insertion_point(class_scope:openxc.PassthroughModeControlCommand)

class AcceptanceFilterBypassCommand(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _ACCEPTANCEFILTERBYPASSCOMMAND
  
  # @@protoc_insertion_point(class_scope:openxc.AcceptanceFilterBypassCommand)

class PayloadFormatCommand(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _PAYLOADFORMATCOMMAND
  
  # @@protoc_insertion_point(class_scope:openxc.PayloadFormatCommand)

class PredefinedObd2RequestsCommand(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _PREDEFINEDOBD2REQUESTSCOMMAND
  
  # @@protoc_insertion_point(class_scope:openxc.PredefinedObd2RequestsCommand)

class NetworkOperatorSettings(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  
  class NetworkDescriptor(message.Message):
    __metaclass__ = reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _NETWORKOPERATORSETTINGS_NETWORKDESCRIPTOR
    
    # @@protoc_insertion_point(class_scope:openxc.NetworkOperatorSettings.NetworkDescriptor)
  DESCRIPTOR = _NETWORKOPERATORSETTINGS
  
  # @@protoc_insertion_point(class_scope:openxc.NetworkOperatorSettings)

class NetworkDataSettings(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _NETWORKDATASETTINGS
  
  # @@protoc_insertion_point(class_scope:openxc.NetworkDataSettings)

class ServerConnectSettings(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SERVERCONNECTSETTINGS
  
  # @@protoc_insertion_point(class_scope:openxc.ServerConnectSettings)

class ModemConfigurationCommand(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _MODEMCONFIGURATIONCOMMAND
  
  # @@protoc_insertion_point(class_scope:openxc.ModemConfigurationCommand)

class RTCConfigurationCommand(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _RTCCONFIGURATIONCOMMAND
  
  # @@protoc_insertion_point(class_scope:openxc.RTCConfigurationCommand)

class CommandResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _COMMANDRESPONSE
  
  # @@protoc_insertion_point(class_scope:openxc.CommandResponse)

class DiagnosticRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _DIAGNOSTICREQUEST
  
  # @@protoc_insertion_point(class_scope:openxc.DiagnosticRequest)

class DiagnosticResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _DIAGNOSTICRESPONSE
  
  # @@protoc_insertion_point(class_scope:openxc.DiagnosticResponse)

class DynamicField(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _DYNAMICFIELD
  
  # @@protoc_insertion_point(class_scope:openxc.DynamicField)

class SimpleMessage(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SIMPLEMESSAGE
  
  # @@protoc_insertion_point(class_scope:openxc.SimpleMessage)

# @@protoc_insertion_point(module_scope)



"""
Current OpenXC version constant.

This functionality is contained in its own module to prevent circular import
problems with ``__init__.py`` (which is loaded by setup.py during installation,
which in turn needs access to this version information.)
"""

VERSION = (0, 13, 0)

__version__ = '.'.join(map(str, VERSION))

def get_version():
    return __version__



"""A data sink implementation for the core listener notification service of
:class:`openxc.vehicle.Vehicle`.
"""
from threading import Thread
from collections import defaultdict
import logging

from openxc.measurements import Measurement, UnrecognizedMeasurementError
from .queued import QueuedSink

LOG = logging.getLogger(__name__)


class MeasurementNotifierSink(QueuedSink):
    """Notify previously registered callbacks whenever measurements of a certian
    type have been received.

    This data sink is the core of the asynchronous interface of
    :class:`openxc.vehicle.Vehicle.`
    """
    def __init__(self):
        super(MeasurementNotifierSink, self).__init__()
        self.callbacks = defaultdict(set)
        self.notifier = self.Notifier(self.queue, self._propagate)

    def register(self, measurement_class, callback):
        """Call the ``callback`` with any new values of ``measurement_class``
        received.
        """
        self.callbacks[Measurement.name_from_class(measurement_class)
                ].add(callback)

    def unregister(self, measurement_class, callback):
        """Stop notifying ``callback`` of new values of ``measurement_class``.

        If the callback wasn't previously registered, this method will have no
        effect.
        """
        self.callbacks[Measurement.name_from_class(measurement_class)
                ].remove(callback)

    def _propagate(self, measurement, **kwargs):
        measurement_callbacks = self.callbacks[measurement.name]
        measurement_callbacks.update(self.callbacks[Measurement.name])
        for callback in measurement_callbacks:
            try:
                callback(measurement, **kwargs)
            except TypeError:
                callback(measurement)

    class Notifier(Thread):
        def __init__(self, queue, callback):
            super(MeasurementNotifierSink.Notifier, self).__init__()
            self.daemon = True
            self.queue = queue
            self.callback = callback
            self.start()

        def run(self):
            while True:
                message, kwargs = self.queue.get()
                try:
                    measurement = Measurement.from_dict(message)
                    self.callback(measurement, **kwargs)
                    self.queue.task_done()
                except UnrecognizedMeasurementError as e:
                    LOG.warn(e)



"""Common functinality for data sinks that work on a queue of incoming
messages.
"""
from .base import DataSink

try:
    from Queue import Queue
except ImportError:
    # Python 3
    from queue import Queue

class QueuedSink(DataSink):
    """Store every message received and any kwargs from the originating data
    source as a tuple in a queue.

    The queue can be reference in subclasses via the `queue` attribute.
    """
    def __init__(self):
        super(QueuedSink, self).__init__()
        self.queue = Queue()

    def receive(self, message, **kwargs):
        """Add the `message` and `kwargs` to the queue."""
        self.queue.put((message, kwargs))



"""A virtual serial port data source."""
from __future__ import absolute_import

import logging

from .base import BytestreamDataSource, DataSourceError

LOG = logging.getLogger(__name__)

try:
    import serial
except ImportError:
    LOG.debug("serial library not installed, can't use serial interface")
    serial = None


class SerialDataSource(BytestreamDataSource):
    """A data source reading from a serial port, which could be implemented
    with a USB to Serial or Bluetooth adapter.
    """
    DEFAULT_PORT = "/dev/ttyUSB0"
    DEFAULT_BAUDRATE = 230400

    def __init__(self, port=None, baudrate=None, **kwargs):
        """Initialize a connection to the serial device.

        Kwargs:
            port - optionally override the default virtual COM port
            baudrate - optionally override the default baudrate

        Raises:
            DataSourceError if the serial device cannot be opened.
        """
        super(SerialDataSource, self).__init__(**kwargs)
        port = port or self.DEFAULT_PORT
        baudrate = baudrate or self.DEFAULT_BAUDRATE

        if serial is None:
            raise DataSourceError("pyserial library is not available")

        try:
            self.device = serial.Serial(port, baudrate, rtscts=True)
        except (OSError, serial.SerialException) as e:
            raise DataSourceError("Unable to open serial device at port "
                    "%s: %s" % (port, e))
        else:
            LOG.debug("Opened serial device at %s", port)

    def read(self):
        return self.device.read()



"""
This module contains the methods for the ``openxc-scanner`` command line
program.

`main` is executed when ``openxc-scanner`` is run, and all other callables in this
module are internal only.
"""
from __future__ import absolute_import

import argparse
from collections import defaultdict

from .common import device_options, configure_logging, select_device

TESTER_PRESENT_MODE = 0x3e
TESTER_PRESENT_PAYLOAD = bytearray([0])

def scan(controller, bus=None, message_id=None):
    message_ids = []
    if message_id is not None:
        message_ids.append(message_id)
    else:
        # using 11-bit IDs
        message_ids = range(0, 0x7ff + 1)

    print("Sending tester present message to find valid modules arb IDs")
    active_modules = set()
    for arb_id in message_ids:
        response = controller.create_diagnostic_request(arb_id, TESTER_PRESENT_MODE,
                bus=bus, wait_for_first_response=True,
                payload=TESTER_PRESENT_PAYLOAD)
        if response is not None:
            print("0x%x responded to tester present: %s" % (arb_id, response))
            active_modules.add(arb_id)

    # Scan for active services on each active module by sending blank requests
    print("Scanning for services on active modules")
    active_modes = defaultdict(list)
    for active_module in active_modules:
        controller.create_diagnostic_request(active_module, TESTER_PRESENT_MODE,
                bus=bus, frequency=1, wait_for_first_response=True,
                payload=TESTER_PRESENT_PAYLOAD)
        # TODO don't really care about response, but need to wait before sending
        # the next request or we will get a pipe error on USB

        print("Scanning services on 0x%x" % active_module)
        for mode in range(1, 0xff):
            # TODO should we be sending blank requests to each mode, or should
            # we send tester present with the service ID specified?
            response = controller.create_diagnostic_request(active_module, mode, bus=bus,
                    wait_for_first_response=True)
            if response is not None:
                print("0x%x responded on service 0x%x" % (active_module, mode))
                # TODO make sure response isn't negative
                active_modes[active_module].append(mode)

        controller.create_diagnostic_request(active_module, TESTER_PRESENT_MODE, bus=bus,
                frequency=0)

    # Scan for what each mode can do and what data it can return by fuzzing the
    # payloads
    print("Fuzzing the valid modes on acitve modules to see what happens")
    for arb_id, active_modes in active_modes.iteritems():
        controller.create_diagnostic_request(arb_id, TESTER_PRESENT_MODE, bus=bus,
                frequency=1, payload=TESTER_PRESENT_PAYLOAD)

        for mode in active_modes:
            # TODO how to support more payloads efficiently?
            for payload in range(0, 0xfffff):
                response = controller.create_diagnostic_request(arb_id, mode,
                        bus=bus, payload=payload, wait_for_first_response=True)
                if response is not None:
                    # TODO make sure response isn't negative
                    # TODO print out something?
                    print("0x%x responded to mode 0x%x request with payload 0x%x with: %s" % (arb_id, mode, payload, response))

        controller.create_diagnostic_request(arb_id, TESTER_PRESENT_MODE, bus=bus,
                frequency=0)

def parse_options():
    parser = argparse.ArgumentParser(description="Send diagnostic message requests to an attached VI",
            parents=[device_options()])
    parser.add_argument("--bus", help="CAN bus controller address to send on")
    parser.add_argument("--message-id", help="CAN message ID for the request")

    return parser.parse_args()


def main():
    configure_logging()
    arguments = parse_options()

    controller_class, controller_kwargs = select_device(arguments)
    controller = controller_class(**controller_kwargs)
    controller.start()

    scan(controller, arguments.bus, arguments.message_id)
